<?php
/*
	+-----------------------------------------------------------------------------+
	| GROUP-E collaboration software                                              |
	+-----------------------------------------------------------------------------+
	| Copyright (c) 2002-2005 group-e,                                            |
	|         Endo7 GmbH/Srl                                                      |
	|	  via-Goethe-strasse 34a                                              |
	|	  39100 Bozen/Bolzano                                                 |
	|	  ITALIEN/ITALIA                                                      |
	|         contact@endo7.com                                                   |
	| This program is free software; you can redistribute it and/or               |
	| modify it under the terms of the GNU General Public License                 |
	| as published by the Free Software Foundation; either version 2              |
	| of the License, or (at your option) any later version.                      |
	|                                                                             |
	| This program is distributed in the hope that it will be useful,             |
	| but WITHOUT ANY WARRANTY; without even the implied warranty of              |
	| MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               |
	| GNU General Public License for more details.                                |
	|                                                                             |
	| You should have received a copy of the GNU General Public License           |
	| along with this program; if not, write to the Free Software                 |
	| Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. |
        | http://www.fsf.org/                                                         |
	+-----------------------------------------------------------------------------+
*/
class AddrLdap {
	var $_ds;
	var $_error;
	var $_sort;
	var $addr_classes=array(
			'top',
			'Person',
			'inetOrgPerson',
	);
	var $_fixed_trans_fields=array(
		'Name'=>'cn',
		'TimeCreate'=>'createTimestamp',
		'TimeModify'=>'modifyTimestamp',
	);
	function _connect() {
		$this->_ds=false;
		$this->_ds=ldap_connect($this->getCfg('cfg/ldap/host'), $this->getCfg('cfg/ldap/port'));
		if ($this->_ds) {
			@ldap_set_option($this->_ds, LDAP_OPT_PROTOCOL_VERSION, 3);
   			if ($this->getCfg('cfg/ldap/tls')) @ldap_start_tls($this->_ds);
		}
		return $this->_ds;
	}
	function getError() {
		return "LDAP: ".@ldap_error($this->_ds);
	}
	function _bind($binddn = NULL ,$password = NULL) {
		$this->_connect();
		if (!$binddn) {
			return ldap_bind($this->_ds);
		} else {
			return ldap_bind($this->_ds, $this->utf8encode($binddn), $this->utf8encode($password));
		}
	}
	function _add($dn,$add) {
		//debug($dn);
		//debug($add);
		return ldap_add($this->_ds,$this->utf8encode($dn),$this->utf8encode($add));
	}
	function _rename($dn,$newRdn,$newParent,$del=true) {
		return ldap_rename($this->_ds, $this->utf8encode($dn), $this->utf8encode($newRdn), $this->utf8encode($newParent), $del);
	}
	function _del($dn) {
		return ldap_delete($this->_ds,$this->utf8encode($dn));
	}
	function _mod_add($dn,$add) {
		//debug($dn);
		//debug($add);
		return ldap_mod_add($this->_ds,$this->utf8encode($dn),$this->utf8encode($add));
	}
	function _mod_rep($dn,$add) {
		//debug($dn);
		//debug($add);
		return ldap_mod_replace($this->_ds,$this->utf8encode($dn),$this->utf8encode($add));
	}
	function _mod_del($dn,$del) {
		//debug($dn);
		//debug($del);
		return ldap_mod_del($this->_ds,$this->utf8encode($dn),$this->utf8encode($del));
	}
	function _search($basedn,$search,$attr,$sub=0) {
		//debug($dn);
		//debug($del);
		if ($sub) {
			return ldap_search($this->_ds,$basedn,$search,$attr,0,0,0);
		} else {
			return ldap_list($this->_ds,$basedn,$search,$attr,0,0,0);
		}
	}
	function _close() {
		@ldap_close($this->_ds);
	}
	function ldapEsc($value) {
		if (is_array($value)) {
			foreach ($value as $key=>$val) {
				$value[$key]=$this->ldapEsc($val);
			}
		} else {
			$tras=array('*'=>'\\2a','('=> '\\28',')'=>'\\29','\\'=>'\\5c','\0'=>'\\00');
			$value=strtr($value,$tras);
		}
		return $value;
	}
	function ldapEscDN($value) {
		if (is_array($value)) {
			foreach ($value as $key=>$val) {
				$value[$key]=$this->ldapEscDN($val);
			}
		} else {
			$tras=array(','=>'\,','+'=>'\+');
			$value=strtr($value,$tras);
		}
		return $value;
	}
	function utf8encode($value) {
		if (is_array($value)) {
			foreach ($value as $key=>$val) {
				$value[$key]=$this->utf8encode($val);
			}
		} else {
			$value=Framework::convertCharset($value,GE_CHARSET,'UTF-8');
		}
		return $value;
	}
	function utf8decode($value) {
		if (is_array($value)) {
			foreach ($value as $key=>$val) {
				$value[$key]=$this->utf8decode($val);
			}
		} else {
			$value=Framework::convertCharset($value,'UTF-8',GE_CHARSET);
		}
		return $value;
	}

	function searchAddrLdap($b,$order,$searchAddr=NULL,$searchNode=NULL,$searchHot=NULL,$searchProject=NULL,$use_limit=false) {
		$cfg=array(
			'cfg/ldap/host'=>'10.0.0.32',
			'cfg/ldap/port'=>389,
			'cfg/ldap/tls'=>0,
			'cfg/ldap/basedn'=>'ou=AdressBook,dc=manni,dc=endo7,dc=com',
		);
		$this->setCfg($cfg);
		$b=intval($b);
		$filter=array('(objectClass=inetOrgPerson)');
		if ($this->_bind()) {
			$fields=$this->getlistFields();
			if (!in_array('Vorname',$fields)) $fields[]='Vorname';
			if (!in_array('Nachname',$fields)) $fields[]='Nachname';
			if (!in_array('Firma',$fields)) $fields[]='Firma';
			if (!in_array('Name',$fields)) $fields[]='Name';
			$ld_fields=$this->_addrFieldsToLdap($fields);
			$ld_fields[]='entryUUID';
			$ld_fields[]='createTimestamp';
			$ld_fields[]='modifyTimestamp';
			if (!empty($searchAddr)) {
				$tmp=$this->_sqlToLdap($searchAddr);
				if ($tmp) $filter[]=$tmp;
			}
			if(count($filter)>1) {
				$ld_filter='(&'.implode('',$filter).')';
			} else {
				$ld_filter=implode('',$filter);
			}
			//debug($ld_filter);
			$sr=$this->_search($this->getCfg('cfg/ldap/basedn'),$ld_filter,$ld_fields,1);
			if (is_array($order)) {
				$order=array_reverse($order,true);
				foreach ($order as $key=>$val) {
					$key=str_replace('AddrMeta.','',$key);
					//debug($key);
					$oF=$this->_addrFieldsToLdap($key);
					if ($oF) {
					//debug($oF);
						ldap_sort($this->_ds,$sr,$oF);
					}
				}
			}
			$entrys=ldap_get_entries($this->_ds,$sr);
			for ($i=$b;$i<$b+$this->getLimit();$i++) {
				if (!isset($entrys[$i])) break;
				$ID=$this->getLdapID($entrys[$i]);
				$result[$ID]=new AObj_addr($ID,$this->fetchLdapAddrObject($entrys[$i],$fields),0,$this->fetchLdapAddrHot($ID),1);
			}
			return $result;
		} else {
			debug($this->getError());
		}
	}
	function fetchLdapAddrObject(&$entry,&$fields) {
		foreach ($fields as $key) {
			$val=strtolower($this->_addrFieldsToLdap($key));
			if ($val && $entry[$val]['count']) {
				$data=array();
				for ($j=0;$j<$entry[$val]['count'];$j++) {
					$data[]=$entry[$val][$j];
					$result[$key]=$this->AddrImplode($data);
				}
			} else {
				$result[$key]='';
			}
		}
		$result['UserID']=0;
		$result['Users']='';
		$result['Groups']='';
		$result['Trash']=0;
		$result['TrashIDs']='';
		$result['TimeCreate']=0;
		$result['TimeModify']=0;
		$result['UserCreate']=0;
		$result['UserModify']=0;
		return $result;
	}
	function &fetchLdapAddrHot($ID) {
		$result=array(
			'Favorite'=>0,
			'TimeLast'=>0,
			'Counter'=>0,
		);
		return $result;
	}
	function getLdapID(&$entry) {
		return $entry['entryuuid'][0];
	}
	function _addrFieldsToLdap($fields) {
		if (is_array($fields)) {
			$result=array();
			foreach ($fields as $val) {
				if (isset($this->_ldap_field_transform[$val])) {
					$result[]=$this->_ldap_field_transform[$val];
				} elseif (isset($this->_fixed_trans_fields[$val])) {
					$result[]=$this->_fixed_trans_fields[$val];
				}
			}
			return $result;
		} else {
			$result='';
			if (isset($this->_ldap_field_transform[$fields])) {
				$result=$this->_ldap_field_transform[$fields];
			} elseif (isset($this->_fixed_trans_fields[$fields])) {
				$result=$this->_fixed_trans_fields[$fields];
			}
			return $result;
		}
	}
	function _sqlToLdap($searchQuery) {
		$where=array();
		if (is_array($searchQuery)) {
			foreach ($searchQuery as $key=>$val) {
				$Field=$this->_addrFieldsToLdap($val['Field']);
				$ld_values=is_array($val['Values'])?$val['Values']:array($val['Values']);
				$tmp_val=array();
				switch ($val['Typ']) {
					case '=':
						if (empty($Field)) continue;
						foreach ($ld_values as $ld_val) {
							$tmp_val[]='('.$Field.'='.$this->ldapEsc($ld_val).')';
						}
					break;
					case '!=':
						if (empty($Field)) continue;
						foreach ($ld_values as $ld_val) {
							$tmp_val[]='('.$Field.'='.$this->ldapEsc($ld_val).')';
						}
					break;
					case 'like':
						if (empty($Field)) continue;
						foreach ($ld_values as $ld_val) {
							$tmp_val[]='('.$Field.'='.str_replace('%','*',$this->ldapEsc($ld_val)).')';
						}
					break;
					case 'concat':
						if (empty($Field)) continue;
						$ld_val=implode(' ',$ld_values);
						$tmp_val[]=$this->formatSearchLdap($ld_val,$Field);
					break;
					case '<':
					case '<=':
					case '>':
					case '>=':
						if (empty($Field)) continue;
						foreach ($ld_values as $ld_val) {
							$tmp_val[]='('.$Field.$val['Typ'].$this->ldapEsc($ld_val).')';
						}
					break;
					case 'OR':
						foreach ($ld_values as $tmp_val) {
							$tmp_val[]=$this->getWhere($tmp_val);
						}
					break;
					case 'F=F':
						continue;
					break;
					case 'F!=F':
						continue;
					break;
					case 'SQL':
						continue;
					break;

				}
				if (count($tmp_val)>1) {
					$where[]='(|'.implode('',$tmp_val).')';
				} elseif(!empty($tmp_val)) {
					$where[]=implode('',$tmp_val);
				}
			}
		}
		if (count($where)>1) {
			return '(&'.implode('',$where).')';
		} else {
			return implode('',$where);
		}
	}
	function formatSearchLdap($info,$fields,$use_case=0,$only_words=0) {
		if (preg_match_all("/\s\"(.+)\"\s/U",' '.$info.' ',$result)) {
			$tmp=preg_replace("/\s\"(.+)\"\s/U","",' '.$info.' ');
			$result[2]=explode(' ',preg_replace('/\s{2,}/',' ',trim($tmp)));
			if (!empty($result[2][0])) {
				$sql_info=array_unique(array_merge($result[1],$result[2]));
			} else {
				$sql_info=$result[1];
			}
		} else {
			$info=preg_replace('/\s{2,}/',' ',$info);
			$sql_info=explode(' ',$info);
		}
		$result='(|';
		if (is_array($fields)) {
			foreach ($fields as $field) {
				$sub_res='';
				foreach($sql_info as $val) {
					if ($val[0]=='-' && strlen($val)>1) {
						$val=substr($val,1);
						$prep='(!';
						$after=')';
					} else {
						$prep='';
						$after='';
					}
					$sub_res.=$prep.'('.$field.'=*'.$this->ldapEsc($val).'*)'.$after;
				}
				if (count($sql_info)>1) {
					$result.='(|'.$sub_res.')';
				} else {
					$result.=$sub_res;
				}
			}
		}
		$result.=')';
		return $result;
	}


}
?>