<?php
class LdapObjectClass {
	# The OID of this schema item.
	var $oid;
	# This objectClass' name, ie "inetOrgPerson"
	var $name;
	# Array of objectClass names from which this objectClass inherits
	var $sup_classes;
	# One of STRUCTURAL, ABSTRACT, or AUXILIARY
	var $type;
	# Arrays of attribute names that this objectClass requires
	var $must_attrs;
	# Arrays of attribute names that this objectClass allows, but does not require
	var $may_attrs;
	# Boolean value indicating whether this objectClass is obsolete
	var $is_obsolete;
	# Array of objectClasses which inherit from this one (must be set at runtime explicitly by the caller)
	var $children_objectclasses;
	# The description of this schema item.
	var $description;
	# Initialize the class' member variables
	function initVars() {
		$this->oid = null;
		$this->name = null;
		$this->description = null;
		$this->sup_classes = array();
		$this->type = 'SUP';
		$this->must_attrs = array();
		$this->may_attrs = array();
		$this->is_obsolete = false;
		$this->children_objectclasses = array();
	}
	function setOID($new_oid) {
		$this->oid = $new_oid;
	}
	function setDescription($new_desc) {
		$this->description = $new_desc;
	}
	function getOID() {
		return $this->oid;
	}
	function getDescription() {
		return $this->description;
	}
	function _parse_list($i,$strings,&$attrs) {
		/*
		 * A list starts with a ( followed by a list of attributes separated by $ terminated by )
		 * The first token can therefore be a ( or a (NAME or a (NAME)
		 * The last token can therefore be a ) or NAME)
		 * The last token may be terminate by more than one bracket
		 */
		$string = $strings[$i];
		if (! preg_match('/^\(/',$string)) {
			# A bareword only - can be terminated by a ) if the last item
			if (preg_match('/\)+$/',$string))
				$string = preg_replace('/\)+$/','',$string);

			array_push($attrs,$string);

		} elseif (preg_match('/^\(.*\)$/',$string)) {
			$string = preg_replace('/^\(/','',$string);
			$string = preg_replace('/\)+$/','',$string);
			array_push($attrs,$string);

		} else {
			# Handle the opening cases first
			if ($string == '(') {
				$i++;

			} elseif (preg_match('/^\(./',$string)) {
				$string = preg_replace('/^\(/','',$string);
				array_push ($attrs,$string);
				$i++;
			}

			// Token is either a name, a $ or a ')'
			// NAME can be terminated by one or more ')'
			while (! preg_match('/\)+$/',$strings[$i])) {
				$string = $strings[$i];
				if ($string == '$') {
					$i++;
					continue;
				}

				if (preg_match('/\)$/',$string)) {
					$string = preg_replace('/\)+$/','',$string);
				} else {
					$i++;
				}

				array_push ($attrs,$string);
			}
		}
		sort($attrs);
		return $i;
	}

	/**
	 * Creates a new ObjectClass object given a raw LDAP objectClass string.
	 * @todo: Unfortunately, some schemas are not well defined - eg: dNSDomain. Where the schema definition is not case consistent with the attribute definitions. This causes us some problems, which we need to resolve.
	 */
	function LdapObjectClass($raw_ldap_schema_string) {
		$this->initVars();
		$class = $raw_ldap_schema_string;
		$strings = preg_split('/[\s,]+/',$class,-1,PREG_SPLIT_DELIM_CAPTURE);

		$str_count = count($strings);

		for ($i=0; $i < $str_count; $i++) {

			switch ($strings[$i]) {
				case '(':
					break;

				case 'NAME':
					if ($strings[$i+1]!='(') {
						do {
							$i++;
							if (strlen($this->name) == 0)
								$this->name = $strings[$i];
							else
								$this->name .= ' ' . $strings[$i];

						} while (! preg_match('/\'$/s',$strings[$i]));

					} else {
						$i++;
						do {
							$i++;
							if (strlen($this->name) == 0)
								$this->name = $strings[$i];
							else
								$this->name .= ' ' . $strings[$i];

						} while (! preg_match('/\'$/s',$strings[$i]));

						do {
							$i++;
						} while (! preg_match('/\)+\)?/',$strings[$i]));
					}

					$this->name = preg_replace('/^\'/','',$this->name);
					$this->name = preg_replace('/\'$/','',$this->name);

					break;

				case 'DESC':
					do {
						$i++;
						if (strlen($this->description) == 0)
							$this->description=$this->description . $strings[$i];
						else
							$this->description=$this->description . ' ' . $strings[$i];

					} while (! preg_match('/\'$/s',$strings[$i]));

					break;

				case 'OBSOLETE':
					$this->is_obsolete = TRUE;

					break;

				case 'SUP':
					if ($strings[$i+1] != '(') {
						$i++;
						array_push($this->sup_classes,preg_replace("/'/",'',$strings[$i]));

					} else {
						$i++;
						do {
							$i++;
							if ($strings[$i] != '$')
								array_push($this->sup_classes,preg_replace("/'/",'',$strings[$i]));

						} while (! preg_match('/\)+\)?/',$strings[$i+1]));
					}

					break;

				case 'ABSTRACT':
					$this->type = 'abstract';
					break;

				case 'STRUCTURAL':
					$this->type = 'structural';
					break;

				case 'AUXILIARY':
					$this->type = 'auxiliary';
					break;

				case 'MUST':
					$attrs = array();

					$i = $this->_parse_list(++$i,$strings,$attrs);


					foreach ($attrs as $string) {
						$attr = new LdapObjectClassAttribute($string,$this->name);
						array_push($this->must_attrs,$attr);
					}
					break;

				case 'MAY':
					$attrs = array();

					$i = $this->_parse_list(++$i,$strings,$attrs);


					foreach ($attrs as $string) {
						$attr = new LdapObjectClassAttribute($string,$this->name);
						array_push ($this->may_attrs,$attr);
					}
					break;

				default:
					if (preg_match('/[\d\.]+/i',$strings[$i]) && $i == 1) {
						$this->oid = $strings[$i];

					}
					break;
			}
		}

		$this->description = preg_replace("/^\'/",'',$this->description);
		$this->description = preg_replace("/\'$/",'',$this->description);

	}

	/**
	 * Gets an array of AttributeType objects that entries of this ObjectClass must define.
	 * This differs from getMustAttrNames in that it returns an array of AttributeType objects
	 *
	 * @param array $oclasses An array of ObjectClass objects to use when traversing
	 *             the inheritance tree. This presents some what of a bootstrapping problem
	 *             as we must fetch all objectClasses to determine through inheritance which
	 *             attributes this objectClass requires.
	 * @return array The array of required AttributeType objects.
	 *
	 * @see getMustAttrNames
	 * @see getMayAttrs
	 * @see getMayAttrNames
	 */
	function getMustAttrs($oclasses=null) {
		$all_must_attrs = array();
		$all_must_attrs = $this->must_attrs;
		foreach ($this->sup_classes as $sup_class) {
			if (! is_null($oclasses) && $sup_class != 'top'
				&& isset($oclasses[strtolower($sup_class)])) {
					$sup_class = $oclasses[ strtolower($sup_class)];
					$sup_class_must_attrs = $sup_class->getMustAttrs($oclasses);
					$all_must_attrs = array_merge($sup_class_must_attrs,$all_must_attrs);
			}
		}

		masort($all_must_attrs,'name,source',1);

		# Remove any duplicates
		foreach ($all_must_attrs as $index => $attr)
			if (isset($allattr[$attr->name]))
				unset($all_must_attrs[$index]);
			else
				$allattr[$attr->name] = 1;

		return $all_must_attrs;
	}

	/**
	 * Gets an array of AttributeType objects that entries of this ObjectClass may define.
	 * This differs from getMayAttrNames in that it returns an array of AttributeType objects
	 *
	 * @param array $oclasses An array of ObjectClass objects to use when traversing
	 *             the inheritance tree. This presents some what of a bootstrapping problem
	 *             as we must fetch all objectClasses to determine through inheritance which
	 *             attributes this objectClass provides.
	 * @return array The array of allowed AttributeType objects.
	 *
	 * @see getMustAttrNames
	 * @see getMustAttrs
	 * @see getMayAttrNames
	 * @see AttributeType
	 */
	function getMayAttrs($oclasses=null) {
		$all_may_attrs = array();
		$all_may_attrs = $this->may_attrs;
		foreach ($this->sup_classes as $sup_class_name) {
			if (! is_null($oclasses) && $sup_class_name != 'top'
				&& isset($oclasses[strtolower($sup_class_name)])) {

					$sup_class = $oclasses[strtolower($sup_class_name)];
					$sup_class_may_attrs = $sup_class->getMayAttrs($oclasses);
					$all_may_attrs = array_merge($sup_class_may_attrs,$all_may_attrs);
			}
		}

		masort($all_may_attrs,'name,source',1);

		# Remove any duplicates
		foreach ($all_may_attrs as $index => $attr)
			if (isset($allattr[$attr->name]))
				unset($all_may_attrs[$index]);
			else
				$allattr[$attr->name] = 1;

		return $all_may_attrs;
	}

	/**
	 * Gets an array of attribute names (strings) that entries of this ObjectClass must define.
	 * This differs from getMustAttrs in that it returns an array of strings rather than
	 * array of AttributeType objects
	 *
	 * @param array $oclasses An array of ObjectClass objects to use when traversing
	 *             the inheritance tree. This presents some what of a bootstrapping problem
	 *             as we must fetch all objectClasses to determine through inheritance which
	 *             attributes this objectClass provides.
	 * @return array The array of allowed attribute names (strings).
	 *
	 * @see getMustAttrs
	 * @see getMayAttrs
	 * @see getMayAttrNames
	 */
	function getMustAttrNames($oclasses=null) {
		$attrs = $this->getMustAttrs($oclasses);
		$attr_names = array();

		foreach ($attrs as $attr)
			$attr_names[] = $attr->getName();

		return $attr_names;
	}

	/**
	 * Gets an array of attribute names (strings) that entries of this ObjectClass must define.
	 * This differs from getMayAttrs in that it returns an array of strings rather than
	 * array of AttributeType objects
	 *
	 * @param array $oclasses An array of ObjectClass objects to use when traversing
	 *             the inheritance tree. This presents some what of a bootstrapping problem
	 *             as we must fetch all objectClasses to determine through inheritance which
	 *             attributes this objectClass provides.
	 * @return array The array of allowed attribute names (strings).
	 *
	 * @see getMustAttrs
	 * @see getMayAttrs
	 * @see getMustAttrNames
	 */
	function getMayAttrNames($oclasses=null) {
		$attrs = $this->getMayAttrs($oclasses);
		$attr_names = array();

		foreach ($attrs as $attr)
			$attr_names[] = $attr->getName();

		return $attr_names;
	}

	/**
	 * Adds an objectClass to the list of objectClasses that inherit
	 * from this objectClass.
	 * @param String $object_class_name The name of the objectClass to add
	 * @return bool Returns true on success or false on failure (objectclass already existed for example)
	 */
	function addChildObjectClass($object_class_name) {
		$object_class_name = trim($object_class_name);
		if (! is_array($this->children_objectclasses))
			$this->children_objectclasses = array();

		foreach ($this->children_objectclasses as $existing_objectclass)
			if (strcasecmp($object_class_name,$existing_objectclass) == 0)
				return false;

		$this->children_objectclasses[] = $object_class_name;
		return true;
	}

	/**
	 * Returns the array of objectClass names which inherit from this objectClass.
	 * @return Array Names of objectClasses which inherit from this objectClass.
	 */
	function getChildObjectClasses() {
		return $this->children_objectclasses;
	}

	/**
	 * Gets the name of this objectClass (ie, "inetOrgPerson")
	 * @return string The name of the objectClass
	 */
	function getName() {
		return $this->name;
	}

	/**
	 * Gets the objectClass names from which this objectClass inherits.
	 *
	 * @return array An array of objectClass names (strings)
	 */
	function getSupClasses() {
		return $this->sup_classes;
	}

	/**
	 * Gets the type of this objectClass: STRUCTURAL, ABSTRACT, or AUXILIARY.
	 */
	function getType() {
		return $this->type;
	}

	/**
	 * Gets whether this objectClass is flagged as obsolete by the LDAP server.
	 */
	function getIsObsolete() {
		return $this->is_obsolete;
	}

	/**
	 * Adds the specified array of attributes to this objectClass' list of
	 * MUST attributes. The resulting array of must attributes will contain
	 * unique members.
	 *
	 * @param array $new_must_attrs An array of attribute names (strings) to add.
	 */
	function addMustAttrs($new_must_attrs) {
		if (! is_array($new_must_attrs))
			return;

		if (count($new_must_attrs) == 0)
			return;

		$this->must_attrs = array_values(array_unique(array_merge($this->must_attrs,$new_must_attrs)));
	}

	/**
	 * Behaves identically to addMustAttrs, but it operates on the MAY
	 * attributes of this objectClass.
	 *
	 * @param array $new_may_attrs An array of attribute names (strings) to add.
	 */
	function addMayAttrs($new_may_attrs) {
		if (! is_array($new_may_attrs))
			return;

		if (count($new_may_attrs) == 0)
			return;

		$this->may_attrs = array_values(array_unique(array_merge($this->may_attrs,$new_may_attrs)));
	}
}

/**
 * A simple class for representing AttributeTypes used only by the ObjectClass class.
 * Users should never instantiate this class. It represents an attribute internal to
 * an ObjectClass. If PHP supported inner-classes and variable permissions, this would
 * be interior to class ObjectClass and flagged private. The reason this class is used
 * and not the "real" class AttributeType is because this class supports the notion of
 * a "source" objectClass, meaning that it keeps track of which objectClass originally
 * specified it. This class is therefore used by the class ObjectClass to determine
 * inheritance.
 * @package phpLDAPadmin
 */
class LdapObjectClassAttribute {
 	# This Attribute's name
	var $name;

	# This Attribute's root
	var $source;

	/**
	 * Creates a new LdapObjectClassAttribute with specified name and source objectClass.
	 * @param string $name the name of the new attribute.
	 * @param string $source the name of the ObjectClass which
	 *           specifies this attribute.
	 */
	function LdapObjectClassAttribute($name,$source) {
		$this->name = $name;
		$this->source = $source;
	}

	# Gets this attribute's name
	function getName () {
		return $this->name;
	}

	# Gets the name of the ObjectClass which originally specified this attribute.
	function getSource () {
		return $this->source;
	}
}


?>
