<?php
class Opensearch {
	protected $_Base;
	protected $lastQueryString;
	protected $errorMsg='';
	protected $error=0;
	protected $_Client;
	protected $_extensions=array(
		'pdf', 'epub',
		'txt','csv', 'xml', 'html', 'htm',
		'doc', 'dot', 'docx', 'dotx','odt', 'rtf',
		'xls', 'xlt', 'xlsx', 'xltx','ods',
		'pptx' , 'ppt', 'pps',  'odp',
		'sxw', 'sxc', 'sxi',
		'jpg', 'jpeg',
		'zip','tar','gz','7z'
	);
	public function __construct($Base) {
		$this->_Base=$Base;
		spl_autoload_register(function($class){
			$f='add-on/' . str_replace('\\','/',$class) . '.php';
			if (file_exists($f)) {
				require_once($f);
			}
		});
		$this->_Client=new \Elastica\Client(array(
			'host' => $this->_Base->getCfg('cfg/opensearch/host'),
			'port' => $this->_Base->getCfg('cfg/opensearch/port'),
		),array($this,'clientError'));
		$this->init();
	}
	public function init($recreate=false) {
		$this->_index = $this->_Client->getIndex($this->_Base->getCfg('cfg/opensearch/index')?$this->_Base->getCfg('cfg/opensearch/index'):'groupe');
		if (!$this->_index->exists() || $recreate) {
			$this->_index->create(
				array(),
				true
			);
			$type = $this->_index->getType('ge_doc');
			// Define mapping
			$mapping = new \Elastica\Type\Mapping();
			// Set mapping
			$mapping->setProperties(array(
				'geDocID'=>array('type' => 'string', 'include_in_all' => true,'store'=>true),
				'geDocParentID'=>array('type' => 'integer', 'include_in_all' => FALSE),
				'geDocContentID'=>array('type' => 'integer', 'include_in_all' => FALSE),
				'geDocVersion'=>array('type' => 'integer', 'include_in_all' => FALSE),
				'geDocVersionActive'=>array('type' => 'boolean', 'include_in_all' => FALSE),
				'geDocUser'=>array('type' => 'integer', 'include_in_all' => FALSE),
				'geDocDeleted'=>array('type' => 'integer', 'include_in_all' => FALSE),

				'geDocType'=>array('type' => 'string', 'include_in_all' => FALSE),
				'geTitle_de'=>array('type' => 'string', 'include_in_all' => TRUE),
				'geTitle_it'=>array('type' => 'string', 'include_in_all' => TRUE),
				'geTitle_en'=>array('type' => 'string', 'include_in_all' => TRUE),
				'geTitle_es'=>array('type' => 'string', 'include_in_all' => TRUE),
				'geTitle_fr'=>array('type' => 'string', 'include_in_all' => TRUE),
				'geText'=>array('type' => 'string', 'include_in_all' => TRUE),

				'geTimeCreate'=>array('type' => 'date', 'include_in_all' => FALSE),
				'geTimeModify'=>array('type' => 'date', 'include_in_all' => FALSE),
				'geUserCreate'=>array('type' => 'integer', 'include_in_all' => FALSE),
				'geUserModify'=>array('type' => 'integer', 'include_in_all' => FALSE),

				'gePermsUsers'=>array('type' => 'integer', 'include_in_all' => FALSE,'store'=>true),
				'gePermsGroups'=>array('type' => 'integer', 'include_in_all' => FALSE,'store'=>true),
				'geFileSize'=>array('type' => 'integer', 'include_in_all' => FALSE),
				'geFile'=>array(
					'type' => 'attachment',
					"fields"=>array(
						"geFile" =>array(
							"type" =>"string",    
							"store" =>"yes",     
							"term_vector" =>"with_positions_offsets" 
						),
					)
				),
			));
			// Send mapping to type
			$type->setMapping($mapping);
		}
		//debug($type);
	}
	public function clientError($connection, $e, $client) {
		try {
			syslog(LOG_WARNING,'Elastica ErrorMsg:'.$e->getMessage());
			/*syslog(LOG_WARNING,'Elasitca ErrorRequest:'.$e->getRequest());
			syslog(LOG_WARNING,'Elasitca ErrorResponse:'.json_encode($e->getResponse()->getData()));*/
		} catch (Exception $e) {
			
		}
		$connection->setEnabled(true);
	}
	function &searchDocument (&$rows,$search=array(),$b=0,$limit=0,$order=array()) {
		$result=array();
		$query=new  Elastica\Query();
		$query->setFields(array(
			'geDocID',
			'geDocParentID',
			'geDocContentID',
			'geDocVersion',
			'geDocVersionActive',
			'geDocUser',
			'geDocDeleted',
			'geTitle_'.$this->_Base->_frm->getLang(),
			'geTimeCreate',
			'geTimeModify',
			'geUserCreate',
			'geUserModify',
		));
		$qEmpty=true;
		$qFiltered=new Elastica\Query\Bool();

		if ($search['Info']) {
			$q=new Elastica\Query\QueryString(str_replace('/','\/',$search['Info']));
			$qFiltered->addMust($q);
			$highlight=array(
				'order'=>'score',
				'fields'=>array(
					'geFile'=>array(
						"fragment_size" => 100,
						"number_of_fragments" =>3,
					),
				),
			);
			$query->setHighlight($highlight);
			$qEmpty=false;
		}
		if ($search['UserCreate']) {
			$q=new Elastica\Query\Term();
			$q->setTerm('geUserCreate', $search['UserCreate']);
			$qFiltered->addMust($q);
			$qEmpty=false;
		}
		if ($search['UserModify']) {
			$q=new Elastica\Query\Term();
			$q->setTerm('geUserModify', $search['UserModify']);
			$qFiltered->addMust($q);
			$qEmpty=false;
		}
		if ($search['TimeCreateB'] || $search['TimeCreateE']) {
			$q=new Elastica\Query\Range();
			$q->addField('geTimeCreate',array(
				'gte'=>$search['TimeCreateB']?$search['TimeCreateB']:null,
				'lte'=>$search['TimeCreateE']?$search['TimeCreateE']:null,
			));
			$qFiltered->addMust($q);
			$qEmpty=false;
		}
		if ($search['TimeModifyB'] || $search['TimeModifyE']) {
			$q=new Elastica\Query\Range();
			$q->addField('geTimeModify',array(
				'gte'=>$search['TimeModifyB']?$search['TimeModifyB']:null,
				'lte'=>$search['TimeModifyE']?$search['TimeModifyE']:null,
			));
			$qFiltered->addMust($q);
			$qEmpty=false;
		}
		
		if ($qEmpty) {
			$qFiltered=new Elastica\Query\MatchAll();
		}

		$fAnd = new \Elastica\Filter\BoolAnd();

		

		$q=new Elastica\Filter\Term();
		$q->setTerm('geDocDeleted',1);
		//$fAnd->addFilter($q);
		$fAnd->addFilter(new Elastica\Filter\BoolNot($q));

		
		if (!$search['AllVersions']) {
			$q=new Elastica\Filter\Term();
			$q->setTerm('geDocVersionActive',true);
			$fAnd->addFilter($q);
		}
		
		$fOr = new \Elastica\Filter\BoolOr();
		if ($this->_Base->isRuleUser()) {
			$q=new Elastica\Filter\Terms();
			$q->setTerms('geDocUser',array(0,(int)$this->_Base->_frm->getUser()));
			$fOr->addFilter($q);
		} else {
			$q=new Elastica\Filter\Term();
			$q->setTerm('gePermsUsers',(int)$this->_Base->_frm->getUser());
			$fOr->addFilter($q);
			$q=new Elastica\Filter\Terms();
			$q->setTerms('gePermsGroups',$this->_Base->_frm->getMyGroups());
			$fOr->addFilter($q);
		}
		/*
		foreach($this->_Base->_frm->getMyGroups() as $v) {
			$q=new Elastica\Filter\Term();
			$q->setTerm('gePermsGroup',$v);
			$fOr->addFilter($q);
		    
		}
		*/
		
		$fAnd->addFilter($fOr);


		$fFiltered=new Elastica\Query\Filtered($qFiltered,$fAnd);

		
		$query->setQuery($fFiltered);
		if ($limit) {
			$query->setFrom($b);    // Where to start?
			$query->setLimit($limit);
		}

		foreach($order as $k=>$v) {
			switch($k) {
				case '_Score':
					$query->addSort(array(
						'_score'=>($v<0?"desc":"asc"),
					));
				case 'Version':
				case 'FileMetabase.Version':
					$query->addSort(array(
						'geDocVersion'=>($v<0?"desc":"asc"),
					));
				break;
				default:
					$query->addSort(array(
						'ge'.$k=>($v<0?"desc":"asc"),
					));
				break;
			}
		}
 		//debug(json_encode($query->toArray()));
		$response=$this->_index->search($query);
 		//debug($response);

		$rows = $response->getTotalHits();
		foreach ($response->getResults() as $doc) {
			$id=$doc->geDocID.'_'.$doc->geDocVersion;
			if ($obj=$this->_Base->getObj($doc->geDocID,'',true,$doc->geDocVersion)) {
				$result[$id]=$obj;
				$result[$id]->setMeta('Version',$doc->geDocVersion);
				$result[$id]->setMeta('TimeCreate',$doc->geTimeCreate);
				$result[$id]->setMeta('TimeModify',$doc->geTimeModify);
				$result[$id]->setMeta('UserCreate',$doc->geUserCreate);
				$result[$id]->setMeta('UserModify',$doc->geUserModify);
				$result[$id]->setMeta('Title_'.$this->_Base->_frm->getLang(),$doc->{'geTitle_'.$this->_Base->_frm->getLang()});
				
	
				$result[$id]->setMeta('_Score',$doc->getScore());
				if ($search['Info']) {
					$result[$id]->setMeta('_Highlights',$doc->getHighlights());
					//debug($doc->getHighlights());
				}
			}
		}
		
		return $result;
	}
	public function deleteDocument($obj,$Version=0) {
		if ($obj->getMeta('DocType')=='ext') {
			$type = $this->_index->getType('ge_doc');
			if ($Version) {
				$id=$obj->getID().'_'.$Version;
				try {
					$type->deleteById($id);
				} catch (Exception $e) {
					
				}
			} else {
				try {
					$q=new Elastica\Query\Term();
					$q->setTerm('geDocID', $obj->getID());
					$type->deleteByQuery($q);
				} catch (Exception $e) {

				}
				
			}
		}
	}
	public function updateDocument($obj,$Version=0,$updateFile=0,$reconstruct=0) {
		$documents = array();
		$data = array(
			'geDocID'=>(int)$obj->getID(),
			'geDocParentID'=>(int)$obj->getParentMeta('ParentID'),
			'geDocContentID'=>(int)$obj->getMeta('Content'),
			'geDocVersion'=>(int)$obj->getMeta('Version'),
			'geDocVersionActive'=>true,
			'geDocType'=>(string)$obj->getMeta('DocType'),
			'geDocUser'=>(int)$obj->getParentMeta('UserID'),
			'geDocDeleted'=>(int)$obj->getMeta('Trash'),
			//'geDocDeleted'=>null,

			'geTitle_de'=>$obj->getMeta('Title_de'),
			'geTitle_it'=>$obj->getMeta('Title_it'),
			'geTitle_en'=>$obj->getMeta('Title_en'),
			'geTitle_es'=>$obj->getMeta('Title_es'),
			'geTitle_fr'=>$obj->getMeta('Title_fr'),
			'geText'=>$obj->getMeta('Text'),

			'geTimeCreate'=>(int)$obj->getMeta('TimeCreate'),
			'geTimeModify'=>(int)$obj->getMeta('TimeModify'),
			'geUserCreate'=>(int)$obj->getMeta('UserCreate'),
			'geUserModify'=>(int)$obj->getMeta('UserModify'),

			'gePermsUsers'=>$obj->getParentMeta('UserID')?array((int)$obj->getParentMeta('UserID')):($obj->getMeta('Users')?array_keys(FrameWork::aclToArray($obj->getMeta('Users'))):array()),
			'gePermsGroups'=>$obj->getMeta('Groups')?array_keys(FrameWork::aclToArray($obj->getMeta('Groups'))):array(),
		);
		if ($obj->getMeta('DocType')=='ext') {
			$rows=0;
			if ($Version) {
				$search=array('Version'=>$Version);
			} else {
				$search=array();
			}
			foreach($this->_Base->searchFile($this->_Base->getModul(),$obj->getMeta('Content'),$rows,0,0,$search,'Version DESC') as $File) {
				if ($File['FilePath']) {
					if (file_exists($File['FilePath'])) {
						$fSize=filesize($File['FilePath']);
						$id='ge_'.$data['geDocID'].'_'.$File['Version'];
						$doc = new \Elastica\Document($id, $data);
						$doc->set('geDocVersion',$File['Version']);
						$doc->set('geFilePath',$File['FilePath']);
						$doc->set('geFileSize',$fSize);
						

						if ($File['Version']==$obj->getMeta('Version')) {
							$doc->set('geDocVersionActive',true);
						} else {
							$doc->set('geDocVersionActive',false);
							if ($reconstruct) {
								$doc->set('geTimeCreate',filemtime($File['FilePath']));
								$doc->set('geTimeModify',filectime($File['FilePath']));
							} else {
								$doc->remove('geTimeCreate');
								$doc->remove('geTimeModify');
								$doc->remove('geUserCreate');
								$doc->remove('geUserModify');
								$doc->remove('geTitle_de');
								$doc->remove('geTitle_it');
								$doc->remove('geTitle_en');
								$doc->remove('geTitle_es');
								$doc->remove('geTitle_fr');
							}
						}
						//if ($updateFile && preg_match('/\.(('.implode(')|(',$this->_extension).'))$/i', $File['FilePath']) && $fSize < 100*1024*1024) {
						if ($updateFile && $fSize < 100*1024*1024) {
							$doc->addFile('geFile', $File['FilePath'], $File['MimeType']);
						}
						$documents[]=$doc;
					}
				}
			}
		}
		//	debug('UPDARE');
		if (!empty($documents)) {
			$type = $this->_index->getType('ge_doc');
			$this->error=0;
			foreach($documents as $v) {
				//debug($v);
				$v->setDocAsUpsert(true);
				try {
					$res=$type->updateDocument($v);
					//debug($res);
				} catch (Exception $e) {
					syslog(LOG_WARNING,'Elastica ErrorMsg: Document ('.$v->getId().') '.$e->getMessage());
					$this->errorMsg.='ERROR : "' . $e->getMessage() . '"'."\n";
					$this->error=1;
					//debug($e);
				}
			}
			//$type->addDocuments($documents);
			$this->_index->refresh();
			return $this->error?false:true;
		} else {
			//Hat noch keine Files also ignorieren;
			return true;
		}
	}
}
?>