<?php
/*
	+-----------------------------------------------------------------------------+
	| GROUP-E collaboration software                                              |
	+-----------------------------------------------------------------------------+
	| Copyright (c) 2002-2005 group-e,                                            |
	|         Endo7 GmbH/Srl                                                      |
	|	  via-Goethe-strasse 34a                                              |
	|	  39100 Bozen/Bolzano                                                 |
	|	  ITALIEN/ITALIA                                                      |
	|         contact@endo7.com                                                   |
	| This program is free software; you can redistribute it and/or               |
	| modify it under the terms of the GNU General Public License                 |
	| as published by the Free Software Foundation; either version 2              |
	| of the License, or (at your option) any later version.                      |
	|                                                                             |
	| This program is distributed in the hope that it will be useful,             |
	| but WITHOUT ANY WARRANTY; without even the implied warranty of              |
	| MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               |
	| GNU General Public License for more details.                                |
	|                                                                             |
	| You should have received a copy of the GNU General Public License           |
	| along with this program; if not, write to the Free Software                 |
	| Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. |
        | http://www.fsf.org/                                                         |
	+-----------------------------------------------------------------------------+
*/
class XmlNode {
	var $_name;
  	var $_children;
  	var $_attribs;
	var $_data;
	var $_parent;
  	function &XmlNode ($name,$attribs='',$data='',$parent=-1) {
    		$this->_name = $name;
    		$this->_attribs = $attribs;
		$this->clear_children();
		$this->_data = $data;
		$this->_data_len =strlen($this->_data);
		$this->_parent = $parent;
    		return $this;
  	}
	function clear_children() {
		$this->_children = array();
		$this->_children_len = 0;
	}
 	function children_len () {
    		return $this->_children_len;
  	}
	function data_len () {
    		return $this->_data_len;
  	}
	function &get_children($pos=false) {
		if ($pos===false) {
 			return $this->_children;
		} else {
			if ($pos==-1) {
				$pos=$this->children_len()-1;
			}
			if (isset($this->_children[$pos])) {
				return $this->_children[$pos];
			}
		}
  	}
  	function &get_data () {
		return $this->_data;
  	}
	function set_data($data) {
		$this->_data = $data;
		$this->_data_len =strlen($this->_data);
	}
	function add_data($data) {
		$this->_data .= $data;
		$this->_data_len =strlen($this->_data);
	}
	function set_attribs ($attribs) {
		$this->_attribs = $attribs;
	}
	function set_attrib ($key,$val) {
		$this->_attribs[$key] = $val;
	}
	function unset_attrib($key) {
		unset($this->_attribs[$key]);
	}
	function &add_child (&$xml_node,$pos=-1) {
		$this->_children_len+=1;
		if ($pos==-1) {
			$this->_children[] = &$xml_node;
		} else {
			array_splice ($this->_children,$pos,0,array(&$xml_node));
		}
		$xml_node->set_parent($this);
		return $xml_node;
	}
	function &delete_child($id=-1,$pos=-1) {
		if ($pos==-1) {
			$pos=$this->get_child_pos($id);
		}
		if ($pos!=-1) {
			$this->_children_len--;
			return reset(array_splice ($this->_children,$pos,1));
		}
	}
	function get_child_pos($id) {
		for ($i=0;$i<$this->_children_len;$i++) {
			if ($this->_children[$i]->get_id()==$id) {
				return $i;
			}
		}
		return -1;
	}
	function move_child($from_pos,$to_pos) {
		if ($from_pos!=$to_pos) {
			//echo "<pre>";
			$tmp=&$this->_children[$from_pos];
			//print_r($this->_children);
			array_splice ($this->_children,$from_pos,1);
			//print_r($this->_children);
			array_splice ($this->_children,$to_pos,0,array(&$tmp));
			//print_r($this->_children);
		}
	}
	function get_name () {
		return $this->_name;
	}
	function get_attribs($name=0) {
		if ($name) {
 			return $this->_attribs[$name];
		} else {
			return $this->_attribs;
		}
	}
	function get_id () {
		return $this->_attribs["id"];
	}
	function &get_parent () {
		return $this->_parent;
	}
	function set_parent (&$parent) {
		$this->_parent=&$parent;
	}

}
class Xml {
	var $_nodes;
	var $_next_id;

	function Xml() {
		$this->_nodes=array();
		$this->_next_id=0;
	}
	function clearAll() {
		while (list($key)=each($this->_nodes)) {
			$obj=& $this->_nodes[$key];
			$obj->_name = '';
			$obj->_attribs = '';
			$obj->clear_children();
			$obj->_data = '';
			$obj->_data_len ='';
			$obj->_parent = '';
 		}
		$this->_nodes=array();
		$this->_next_id=0;
	}
	function &new_child ($parent_id,$name, $attribs='', $data = '',$pos=-1) {
		$attribs["id"]=$this->_next_id;
		$this->_nodes[$this->_next_id]=& new XmlNode ($name,$attribs,$data,$parent_id);
		$this->_nodes[$parent_id]->add_child($this->_nodes[$this->_next_id],$pos);
		$this->_next_id++;
		return ($this->_next_id-1);
	}
	function &new_root ($name, $attribs='', $data = '') {
		$attribs["id"]=$this->_next_id;
		$this->_nodes[$this->_next_id]=& new XmlNode ($name,$attribs,$data);
		$this->_next_id++;
		return ($this->_next_id-1);
	}
	function is_node($id) {
		return isset($this->_nodes[$id]);
	}
	function set_attribs ($id,$attribs) {
		if (isset($this->_nodes[$id])) {
			foreach ($attribs as $key=>$val) {
				$this->_nodes[$id]->set_attrib($key,$val);
			}
		}
	}
	function unset_attribs ($id,$attribs) {
		if (isset($this->_nodes[$id])) {
			if (is_array($attribs)) {
				foreach ($attribs as $val) {
					$this->_nodes[$id]->unset_attrib($val);
				}
			} else {
				$this->_nodes[$id]->unset_attrib($attribs);
			}
		}
	}
	function set_all_attribs ($id,$attribs) {
		if (isset($this->_nodes[$id])) {
			$this->_nodes[$id]->set_attribs($attribs);
		}
	}
	function get_attribs($id,$name=0) {
		if (isset($this->_nodes[$id])) {
			return $this->_nodes[$id]->get_attribs($name);
		}
	}
	function &get_node($id) {
		if (isset($this->_nodes[$id])) {
			return $this->_nodes[$id];
		}
	}
	function &get_children($id) {
		if (isset($this->_nodes[$id])) {
			return $this->_nodes[$id]->get_children();
		}
	}

	function &get_child($parent_id,$pos) {
		if (isset($this->_nodes[$parent_id])) {
			return $this->_nodes[$parent_id]->get_children($pos);
		}
	}
	function children_len($id) {
		if (isset($this->_nodes[$id])) {
			return $this->_nodes[$id]->children_len();
		}
	}
	function get_child_pos($parent,$id) {
		if ($this->is_node($parent)) {
			return $this->_nodes[$parent]->get_child_pos($id);
		} else {
			return -1;
		}
	}
	function &delete_node($id) {
		if ($this->is_node($id)) {
			$parent=&$this->_nodes[$id]->get_parent();
			if (is_object($parent)) {
				$parent_id=$parent->get_id();
				unset($this->_nodes[$id]);
				return $this->_nodes[$parent_id]->delete_child($id);
			}
		}
	}
	function get_recur_data($id,$len=-1) {
		if (isset($this->_nodes[$id])) {
			//echo $this->_nodes[$id]->get_name().'<br>';
			if ($this->_nodes[$id]->data_len()) {
				$data.=$this->_nodes[$id]->get_data().' ';
			}
			if ($this->_nodes[$id]->children_len()) {
				foreach ($this->_nodes[$id]->get_children() as $child) {
					$data.=$this->get_recur_data($child->get_id(),$len);
					if ($len!=-1 && strlen($data)>=$len) {
						$data=substr($data,0,$len);
						break;
					}
				}

			}
			if ($len!=-1 && strlen($data)>$len) {
				$data=substr($data,0,$len);
			}
			return $data;
		}
	}
	function move_node($id,$to_parent,$to_pos=-1,$subtract=0) {
		if ($this->is_node($id)) {
			$parent=&$this->_nodes[$id]->get_parent();
			if (is_object($parent)) {
				$from_parent=$parent->get_id();

				if ($from_parent!=$to_parent) {
					if (isset($this->_nodes[$to_parent])) {
						$node=&$this->_nodes[$to_parent]->add_child($this->_nodes[$from_parent]->delete_child($id),$to_pos);
						//$node->set_parent($this->_nodes[$to_parent]);
					}
				} else {
					if (isset($this->_nodes[$to_parent])) {
						$from_pos=$this->_nodes[$to_parent]->get_child_pos($id);
						if ($subtract && $from_pos<$to_pos) $to_pos--;
						$this->_nodes[$to_parent]->move_child($from_pos,$to_pos);
					}

				}
			}
		}
	}
	function parseText ($xml_text,$recompute=false) {
		$xml_parser = xml_parser_create ('UTF-8');
		xml_parser_set_option ($xml_parser, XML_OPTION_CASE_FOLDING, FALSE);
		xml_parser_set_option ($xml_parser, XML_OPTION_TARGET_ENCODING,GE_CHARSET);
		if (!xml_parse_into_struct ($xml_parser,Framework::convertCharset($xml_text,GE_CHARSET,'UTF-8') , $values, $index)) {
			$this->xmlerror=array('line'=>xml_get_current_line_number($xml_parser),'col'=>xml_get_current_byte_index($xml_parser),'code'=>xml_get_error_code($xml_parser),'value'=> xml_error_string (xml_get_error_code($xml_parser)));
				debug($this->xmlerror);
			return false;
		}
		xml_parser_free($xml_parser);
		unset($current_id);
		$stack=array();
		$this->_nodes=array();
		$this->_next_id=0;
		foreach ($values as $val) {

			switch ($val["type"]) {
				case "open":
					if (!$recompute && $val["attributes"]["id"]) {
						$id=$val["attributes"]["id"];
					} else {
						$id=$this->_next_id;
					}
					$val["attributes"]["id"]=$id;
					$this->_nodes[$id]= & new XmlNode ($val["tag"],$val["attributes"],$val["value"]);
					if (isset($current_id)) {
						$this->_nodes[$current_id]->add_child ($this->_nodes[$id]);
						//$this->_nodes[$id]->set_parent($this->_nodes[$current_id]);
					}
					$current_id=$id;
					array_push($stack,$current_id);
					$this->_next_id=max($this->_next_id,($id+1));
					break;
				case "complete":
					if (!$recompute && $val["attributes"]["id"]) {
						$id=$val["attributes"]["id"];
					} else {
						$id=$this->_next_id;
					}
					$val["attributes"]["id"]=$id;
					$this->_nodes[$id]= & new XmlNode ($val["tag"],$val["attributes"],$val["value"]);
					if (isset($current_id)) {
						$this->_nodes[$current_id]->add_child ($this->_nodes[$id]);
						//$this->_nodes[$id]->set_parent($this->_nodes[$current_id]);
					}
					$this->_next_id=max($this->_next_id,($id+1));
				break;
				case "close":
					array_pop($stack);
					$current_id=end($stack);
					break;
				case "cdata":
					break;
				default:
					break;
			}
		}
		unset($stack,$values,$index);
		return true;
	}

	function addText ($xml_text,$current_id=-1,$replace=false,$current_pos=-1) {
		$xml_parser = xml_parser_create ('UTF-8');
		xml_parser_set_option ($xml_parser, XML_OPTION_CASE_FOLDING, FALSE);
		xml_parser_set_option ($xml_parser, XML_OPTION_TARGET_ENCODING,GE_CHARSET);
		if (!xml_parse_into_struct ($xml_parser,Framework::convertCharset($xml_text,GE_CHARSET,'UTF-8') , $values, $index)) {
			$cont_array=explode("\n",$xml_text);
                	$error_line=xml_get_current_line_number($xml_parser);
               	 	$error_line_text=$cont_array[$error_line-1];
                	if ($error_line==count($cont_array)) $error_code=1;
                	else $error_code=2;
                	$this->xmlerror=array('line'=>$error_line,'linetext'=>$error_line_text,'code'=>$error_code);
			return false;
		}
		xml_parser_free($xml_parser);

		if ($current_id==-1) {
			$this->_next_id=0;
			$this->_nodes=array();
			$stack=array();
			$pstack=array();
		} else {
			if (!$this->is_node($current_id)) return -1;
			if ($replace) {
				$this->_nodes[$current_id]->set_data('');
				$this->_nodes[$current_id]->clear_children();
			}
			$stack=array($current_id);
			$pstack=array($current_pos);

		}
		//echo '<pre>';
		//echo FrameWork::entities(print_r($values,1));
		foreach ($values as $val) {
			//echo 'STACKPOS:'.$current_id.':'.$val["tag"].':'.$val["type"].':'.implode(',',$val["attributes"]).'<br>';
			switch ($val["type"]) {
				case "complete":
				case "open":
					if ($val["tag"]!='obj:parent') {
						$val["attributes"]["id"]=$this->_next_id;
						$this->_nodes[$this->_next_id]= & new XmlNode ($val["tag"],$val["attributes"],NULL);
						if ($current_id!=-1) {
							//echo $current_id.':Add '.$this->_next_id.'<br>';
							$this->_nodes[$current_id]->add_child ($this->_nodes[$this->_next_id],$current_pos);
							//$this->_nodes[$this->_next_id]->set_parent($this->_nodes[$current_id]);
						}
						$current_id=$this->_next_id;
						$current_pos=-1;
						array_push($stack,$current_id);
						array_push($pstack,$current_pos);
						$this->_next_id++;
					}
					if (strlen($val["value"])) {
						//echo $current_id.':Has Data<br>';
						$attribs=array('id'=>$this->_next_id);
						$this->_nodes[$this->_next_id]= & new XmlNode ('_text',$attribs,$val["value"]);
						$this->_nodes[$current_id]->add_child ($this->_nodes[$this->_next_id]);
						//$this->_nodes[$this->_next_id]->set_parent($this->_nodes[$current_id]);
						$this->_next_id++;
					}
					If ($val["type"]!='complete') {
						break;
					}
				case "close":
					if ($val["tag"]!='obj:parent') {
						array_pop($stack);
						$current_id=end($stack);
						array_pop($pstack);
						$current_pos=end($pstack);
					}
					break;
				case "cdata":
					if (strlen($val[value])) {
						if ($pos=$this->_nodes[$current_id]->children_len()) {
							$last_child=$this->_nodes[$current_id]->get_children($pos-1);
						} else $last_child=false;

						if (is_object($last_child) && $last_child->get_name()=='_text') {
							$this->_nodes[$last_child->get_id()]->add_data($val[value]);
						} else {
							$attribs=array('id'=>$this->_next_id);
							$this->_nodes[$this->_next_id]= & new XmlNode ('_text',$attribs,$val["value"]);
							$this->_nodes[$current_id]->add_child ($this->_nodes[$this->_next_id]);
							//$this->_nodes[$this->_next_id]->set_parent($this->_nodes[$current_id]);
							$this->_next_id++;
						}
					}
				default:
					break;
			}
		}
		unset($stack,$stackpos,$values,$index);
		return true;
	}
	function getText ($id=0,$recurse=1,$indent=" ",$newline="\n",$ig_text=false) {
		$this->depth=0;
		return $this->printTag($id,$recurse,$indent,$newline,$ig_text);
	}
	function printTag($id,$recurse,$indent=" ",$newline="\n",$ig_text=false) {
		//echo $this->_nodes[$id]->get_name().'<br>';
		$Name=$this->_nodes[$id]->get_name();
		if (!$ig_text || $Name!='_text' ) {
			$result .=str_repeat($indent,$this->depth)."<" . $Name;
			//echo FrameWork::entities($result).'<br>';
			$attribs=$this->_nodes[$id]->get_attribs();
			if (is_array ($attribs)) {
				foreach ($attribs as $key => $val) {
					$result .= " " . $key . "='" . htmlspecialchars($val,ENT_QUOTES) . "'";
				}
			}
		}
		if ($this->_nodes[$id]->children_len() || $this->_nodes[$id]->data_len()) {
			if (!$ig_text || $Name!='_text' ) $result .= ">";
			if ($this->_nodes[$id]->children_len() && $recurse) {
				$result.=$newline;
				foreach ($this->_nodes[$id]->get_children() as $child) {
					$this->depth++;
					$result .= $this->printTag($child->get_id(),$recurse,$indent,$newline,$ig_text);
					$this->depth--;
				}
				$result .=str_repeat($indent,$this->depth);
			}
			if ($this->_nodes[$id]->data_len()) {
				$result .= htmlspecialchars($this->_nodes[$id]->get_data());
			}
			if (!$ig_text || $Name!='_text' ) $result .= "</" . $Name . ">".$newline;
		} else {
			if (!$ig_text || $Name!='_text' ) $result .= " />".$newline;
		}

		return $result;
	}
	function convertEndings($text) {
		$text=str_replace("\r\n","\n",$text);
		$text=str_replace("\r","\n",$text);
		return $text;
	}
	function searchNode($id,$name) {
		$result=array();
		$this->_searchNode($id,$name,$result);
		return $result;
	}
	function _searchNode($id,$name,&$result) {
		if ($this->_nodes[$id]->get_name()==$name) {
			$result[]=$id;
		}
		if ($this->_nodes[$id]->children_len()) {
			foreach ($this->_nodes[$id]->get_children() as $child) {
				$this->_searchNode($child->get_id(),$name,$result);
			}
		}
	}

}
?>
