<?php
/*
	+-----------------------------------------------------------------------------+
	| GROUP-E collaboration software                                              |
	+-----------------------------------------------------------------------------+
	| Copyright (c) 2002-2005 group-e,                                            |
	|         Endo7 GmbH/Srl                                                      |
	|	  via-Goethe-strasse 34a                                              |
	|	  39100 Bozen/Bolzano                                                 |
	|	  ITALIEN/ITALIA                                                      |
	|         contact@endo7.com                                                   |
	| This program is free software; you can redistribute it and/or               |
	| modify it under the terms of the GNU General Public License                 |
	| as published by the Free Software Foundation; either version 2              |
	| of the License, or (at your option) any later version.                      |
	|                                                                             |
	| This program is distributed in the hope that it will be useful,             |
	| but WITHOUT ANY WARRANTY; without even the implied warranty of              |
	| MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               |
	| GNU General Public License for more details.                                |
	|                                                                             |
	| You should have received a copy of the GNU General Public License           |
	| along with this program; if not, write to the Free Software                 |
	| Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. |
        | http://www.fsf.org/                                                         |
	+-----------------------------------------------------------------------------+
*/
class SendMail {
	/*
	list of To addresses
	@var    array
	*/
	var $sendto = array();
	/*
	@var    array
	*/
	var $acc = array();
	/*
	@var    array
	*/
	var $abcc = array();
	/*
	paths of attached files
	@var array
	*/
	var $aattach = array();
	/*
	list of message headers
	@var array
	*/
	var $xheaders = array();
	/*
	message priorities referential
	@var array
	*/
	var $priorities = array( '1 (Highest)', '2 (High)', '3 (Normal)', '4 (Low)', '5 (Lowest)' );
	/*
	character set of message
	@var string
	*/
	var $charset = "iso-8859-15";
	var $ctencoding = "8bit";
	var $receipt = 0;
	var $nl = "\r\n";

	function SendMail($nl="\r\n") {
		$this->nl=$nl;
		$this->boundary= "--" . md5( uniqid("myboundary") );
	}


	/*

	Define the subject line of the email
	@param string $subject any monoline string

	*/
	function Subject( $subject ) {
		$this->xheaders['Subject'] = $subject;
	}


	/*

	set the sender of the mail
	@param string $from should be an email address

	*/

	function From( $from ) {
		$this->xheaders['From'] = $from;
	}

	/*
	set the Reply-to header
	@param string $email should be an email address

	*/
	function ReplyTo( $address ) {
		$this->xheaders["Reply-To"] = $address;
	}


	/*
	add a receipt to the mail ie.  a confirmation is returned to the "From" address (or "ReplyTo" if defined)
	when the receiver opens the message.
	@warning this functionality is *not* a standard, thus only some mail clients are compliants.

	*/
	function Receipt() {
		$this->receipt = 1;
	}


	/*
	set the mail recipient
	@param string $to email address, accept both a single address or an array of addresses

	*/
	function To( $to ) {
		if(is_array( $to )) {
			$this->sendto= $to;
		} else {
			$this->sendto[] = $to;
		}
	}


	/*              Cc()
	*              set the CC headers ( carbon copy )
	*              $cc : email address(es), accept both array and string
	*/

	function Cc( $cc ) {
		if(is_array($cc)) {
			$this->acc= $cc;
		} else {
			$this->acc[]= $cc;
		}
	}



	/*              Bcc()
	*              set the Bcc headers ( blank carbon copy ).
	*              $bcc : email address(es), accept both array and string
	*/

	function Bcc( $bcc ) {
		if(is_array($bcc)) {
			$this->abcc= $bcc;
		} else {
			$this->abcc[]= $bcc;
		}
	}

	/*              Body( text [, charset] )
	*              set the body (message) of the mail
	*              define the charset if the message contains extended characters (accents)
	*              default to us-ascii
	*              $mail->Body( "ml en franais avec des accents", "iso-8859-1" );
	*/
	function Body( $body, $charset="" )	{
		$this->body = $body;
		if( $charset != "" ) {
			$this->charset = strtolower($charset);
			if( $this->charset != "us-ascii" )
			$this->ctencoding = "8bit";
		}
	}


	/*              Organization( $org )
	*              set the Organization header
	*/
	function Organization( $org ) {
		if(trim( $org != "" )) {
			$this->xheaders['Organization'] = $org;
		}
	}


	/*              Priority( $priority )
	*              set the mail priority
	*              $priority : integer taken between 1 (highest) and 5 ( lowest )
	*              ex: $mail->Priority(1) ; => Highest
	*/
	function Priority( $priority ) {
		if( ! intval( $priority ) )	return false;
		if( ! isset( $this->priorities[$priority-1]) )	return false;
		$this->xheaders["X-Priority"] = $this->priorities[$priority-1];
		return true;
	}


	/*
	Attach a file to the mail

	@param string $filename : path of the file to attach
	@param string $filetype : MIME-type of the file. default to 'application/x-unknown-content-type'
	@param string $disposition : instruct the Mailclient to display the file if possible ("inline") or always as a link ("attachment") possible values are "inline", "attachment"
	*/

	function Attach( $filename, $filetype = "", $disposition = "attachment" ) {
		if( $filetype == "" ) $filetype = "application/x-unknown-content-type";
		$this->aattach[] = $filename;
		$this->actype[] = $filetype;
		$this->adispo[] = $disposition;
	}

	/*

	Build the email message

	@access protected

	*/
	function BuildMail() {
		// build the headers
		$this->headers = "";
		$this->xheaders["Mime-Version"] = "1.0";
		if( count($this->sendto) > 0 )	$this->xheaders['To'] = implode( ", ", $this->sendto );
		if( count($this->acc) > 0 )	$this->xheaders['Cc'] = implode( ", ", $this->acc );
		if( count($this->abcc) > 0 ) $this->xheaders['Bcc'] = implode( ", ", $this->abcc );
		if( $this->receipt ) {
			if( isset($this->xheaders["Reply-To"] ) ) {
				$this->xheaders["Disposition-Notification-To"] = $this->xheaders["Reply-To"];
			} else {
				$this->xheaders["Disposition-Notification-To"] = $this->xheaders['From'];
			}
		}
		// include attached files
		if( count( $this->aattach ) > 0 ) {
			$this->_build_attachement();
		} else {
			$this->fullBody = $this->body;
			$this->xheaders["Content-Type"] = "text/plain; charset=\"".$this->charset."\"";
			$this->xheaders["Content-Transfer-Encoding"] = $this->ctencoding;
		}
		reset($this->xheaders);
		$nl='';
		foreach ($this->xheaders as $hdr=>$value) {
			if( $hdr != "Subject" ) {
				$this->headers .= $nl."$hdr: ".$this->foldLine($value);
				$nl=$this->nl;
			}
		}
	}
	function foldLine(&$value) {
		//return $value;
		return wordwrap($value,200,"\n ");
	}

	function Send()	{
		$this->BuildMail();
		$from=imap_rfc822_parse_adrlist($this->xheaders['From'],'');
		$res = mail( null, $this->xheaders['Subject'], $this->fullBody,$this->headers,'-f'.$from[0]->mailbox.'@'.$from[0]->host);
		return $res;
	}

	function &getMail() {
		$this->BuildMail();
		$h="Date: ".date('r').$this->nl
		."Subject: ".$this->xheaders['Subject'].$this->nl
        .$this->headers
        .$this->nl.$this->nl;
		Framework::convertLineEndings($h,"\r\n");
		Framework::convertLineEndings($this->fullBody,"\r\n");
		return $h.$this->fullBody."\r\n";
	}

	/*
	check and encode attach file(s) . internal use only
	@access private
	*/

	function _build_attachement() {
		$this->xheaders["Content-Type"] = "multipart/mixed;".$this->nl." boundary=\"$this->boundary\"";

		$this->fullBody = "This is a multi-part message in MIME format.".$this->nl."--$this->boundary".$this->nl;
		$this->fullBody .= "Content-Type: text/plain; charset=\"".$this->charset."\"".$this->nl."Content-Transfer-Encoding: ".$this->ctencoding."".$this->nl . $this->nl . $this->body . $this->nl;
		$ata= array();
		$k=0;

		// for each attached file, do...
		for( $i=0; $i < count( $this->aattach); $i++ ) {
			$filename = $this->aattach[$i];
			$basename = basename($filename);
			$ctype = $this->actype[$i];     // content-type
			$disposition = $this->adispo[$i];
			if(file_exists( $filename) ) {
				$subhdr= "--$this->boundary".$this->nl."Content-type: $ctype;".$this->nl." name=\"".Framework::mail_qenc($basename)."\"".$this->nl."Content-Transfer-Encoding: base64".$this->nl."Content-Disposition: $disposition;".$this->nl."  filename=\"".Framework::mail_qenc($basename)."\"".$this->nl;
				$ata[$k++] = $subhdr;
				// non encoded line length
				$linesz= filesize( $filename)+1;
				$fp= fopen( $filename, 'r' );
				$ata[$k++] = chunk_split(base64_encode(fread( $fp, $linesz)));
				fclose($fp);
			}
		}
		$this->fullBody .= implode($this->nl, $ata);
		$this->fullBody .= "--".$this->boundary."--";
	}


} // class Mail


?>
