<?php
class SambaAclClient {
	var $cfgSmbClient = 'smbcacls';
	var $user='', $pw='', $cfgAuthMode='SMB_AUTH_ARG';
	var $cfgDefaultServer='localhost', $cfgDefaultUser='', $cfgDefaultPassword='';
	var $workgroup='', $server='', $share='', $path='';
	var $name = '';
	var $workgroups=array(), $servers=array(), $shares=array(), $files=array();
	var $debug = 0;
	var $status = '';
	var $parser = array(
		'^REVISION:([0-9]+)$' => 'REVISION',
		'^OWNER:(.*)$' => 'OWNER',
		'^GROUP:(.*)$' => 'GROUP',
		'^ACL:([^:]+):([^\/]+)\/([^\/]+)\/([^\/]+)$' => 'ACL',
		'(NT_STATUS_ACCESS_DENIED)'=>'ERROR',
		'(NT_STATUS_LOGON_FAILURE)'=>'ERROR',
		'(NT_STATUS_BAD_NETWORK_NAME)'=>'ERROR',
	);
	var $_mask = array(
		'0x001f01ff'=>7,
		'0x0012019f'=>3,
		'0x001200a9'=>5,
		'0x00120116'=>2,
		'0x00120089'=>1,
		'0x00120136'=>6,
	);
	var $_sid = array (
		'S-1-1-0'=>'[[everyone]]',
		'S-1-3-0'=>'[[creatorOwner]]',
		'S-1-3-1'=>'[[creatorGroup]]',
	);

	function SambaClient () {

	}
	function setUserName($user) {
		$this->user=$user;
	}
	function setPassword($pw) {
		$this->pw=$pw;
	}

	# path: WORKGROUP/SERVER/SHARE/PATH
	function Go ($path = '') {
		$a = ($path <> '') ? split('/',$path) : array();
		for ($i=0, $ap=array(); $i<count($a); $i++) {
			switch ($i) {
				case 0:
					$this->workgroup = $a[$i];
				break;
				case 1:
					$this->server = $a[$i];
				break;
				case 2:
					$this->share = $a[$i];
				break;
				default:
					$ap[] = $a[$i];
			}
		}
		$this->path = join('/', $ap);
		$this->name = basename($path);
		$this->parent = $this->_DirectoryName($this->path);
		$this->fullPath = $path;
	}

	function getAcl () {
		$this->result = array();
		$this->_SmbClient('get');
		return $this->status;
	}
	function aclToMask($acl) {
		$result=array_search(intval($acl),$this->_mask);
		if (!$result) $result='0x0';
		/*if ($acl&1) $result.='R';
		if ($acl&2) $result.='W';
		if ($acl&4) $result.='X';*/
		return $result;
	}
	function addAcl ($sid,$mask,$flags) {
		$acl='ACL:'.$sid.':ALLOWED/'.$flags.'/'.$mask;
		$this->_SmbClient('add',$acl);
		return $this->status;
	}
	function modifyAcl ($sid,$mask,$flags) {
		$acl='ACL:'.$sid.':ALLOWED/'.$flags.'/'.$mask;
		$this->_SmbClient('modify',$acl);
		return $this->status;
	}
	function setAcl($acl) {
		//debug($acl);
		$this->_SmbClient('set',$acl);
		return $this->status;
	}
	function deleteAcl ($acl) {
		$this->_SmbClient('delete','ACL:'.$acl);
		return $this->status;
	}
	function createAcl($AclObj,$type='file') {
		$txt=array();
		$txt[]='REVISION:1';
		$txt[]='OWNER:'.$AclObj['OWNER'];
		$txt[]='GROUP:'.$AclObj['GROUP'];
		foreach ($AclObj['ACL'] as $sid=>$acl_values) {
			foreach ($acl_values as $acl) {
				if ($type=='file') {
					if ($acl['FLAGS']&8) continue;
					$acl['FLAGS']=0;
				}
				$txt[]='ACL:'.$sid.':ALLOWED/'.$acl['FLAGS'].'/'.$this->aclToMask($acl['MASK']);
			}
		}
		return implode("\n",$txt);
	}
	function _SmbClient ($command='get',$acl=''){
		//debug($command);
		$this->status = '';
		switch ($command) {
			case 'get':
				$smbcmd = escapeshellarg("//{$this->server}/{$this->share}")." ".($this->path?escapeshellarg("{$this->path}"):"''");
				$smbcmd .= ' --numeric';
			break;
			case 'add':
				$smbcmd = escapeshellarg("//{$this->server}/{$this->share}")." ".($this->path?escapeshellarg("{$this->path}"):"''");
				$smbcmd .= ' -a '.escapeshellarg($acl);
			break;
			case 'modify':
				$smbcmd = escapeshellarg("//{$this->server}/{$this->share}")." ".($this->path?escapeshellarg("{$this->path}"):"''");
				$smbcmd .= ' -M '.escapeshellarg($acl);
			break;
			case 'set':
				$smbcmd = escapeshellarg("//{$this->server}/{$this->share}")." ".($this->path?escapeshellarg("{$this->path}"):"''");
				$smbcmd .= ' -S '.escapeshellarg($acl);
			break;
			case 'delete':
				$smbcmd = escapeshellarg("//{$this->server}/{$this->share}")." ".($this->path?escapeshellarg("{$this->path}"):"''");
				$smbcmd .= ' -D '.escapeshellarg($acl);
			break;
		}
		if ($this->user <> '') {
			# not anonymous
			switch ($this->cfgAuthMode) {
				case 'SMB_AUTH_ENV':
					putenv('USER='.$this->user.'%'.$this->pw);
				break;
				case 'SMB_AUTH_ARG':
					$smbcmd .= ' -U '.escapeshellarg($this->user.'%'.$this->pw);
			}
		}
		$cmdline = $this->cfgSmbClient.' '.$smbcmd;
		$cmdline .= ' 2>&1';
		return $this->_ParseSmbClient ($cmdline);
	}

	function _ParseSmbClient ($cmdline, $dumpFile=false) {
		//debug($cmdline);
		$this->result=array();
		$sec_cmdline = str_replace($this->pw, '****', $cmdline);
	//echo $cmdline.'<br>';
		//debugtime('B');
		$output = shell_exec($cmdline);
		//debugtime('E');
		$debug_command = ($this->debug > 1) ? "\n[smbclient]\n{$output}\n[/smbclient]" : "";
		$this->Debug("{$sec_cmdline}{$debug_command}",1);
		//debug($output);
		$lineType = $mode = '';
		foreach (split("\n", $output) as $line) if ($line <> '') {
			$regs = array();
			$lineType = 'SKIP';
			$regs = array();
			//debug($line);
			foreach ($this->parser as $regexp => $type) {
				//debug($regexp);
				if (preg_match('/'.$regexp.'/', $line, $regs)) {
					$lineType = $type;
					break;
				}
			}
			//debug($lineType);
			switch ($lineType) {
				case 'SKIP':
				break;
				case 'REVISION':
					$this->result['REVISION']=$regs[1];
				break;
				case 'OWNER':
					$this->result['OWNER']=$regs[1];
				break;
				case 'GROUP':
					$this->result['GROUP']=$regs[1];
				break;
				case 'ACL':
					if ($regs[4]) {
						$acl=array(
							'TYPE'=>$regs[2],
							'FLAGS'=>intval($regs[3],0),
							'MASK'=>$this->_mask[$regs[4]],
						);
						$this->result['ACL'][$regs[1]][]=$acl;
					}
				break;
				case 'ERROR':
					$this->status = $regs[1];
				break;
				default:
					$this->status = $lineType;
			}
		}
	}

	# returns unix time from smbclient output
	function _ParseTime ($m, $d, $y, $hhiiss) {
		$his= split(':', $hhiiss);
		$im = 1 + strpos("JanFebMarAprMayJunJulAugSepOctNovDec", $m) / 3;
		return mktime($his[0], $his[1], $his[2], $im, $d, $y);
	}
	# I do not like PHP dirname
	function _DirectoryName ($path='') {
		$a = split('/', $path);
		$n = (trim($a[count($a)-1]) == '') ? (count($a)-2) : (count($a)-1);
		for ($dir=array(),$i=0; $i<$n; $i++) $dir[] = $a[$i];
		return join('/',$dir);
	}
	function Debug ($message, $level=0)	{
		if ($level <= $this->debug) {
			foreach (split("\n",$message) as $line) {
				//syslog(LOG_INFO, $this->user.'['.$this->GetIP().']: '.$line);
				echo  $this->user.'['.$this->GetIP().']: '.htmlspecialchars($line).'<br>';
			}
		}
	}
	function GetIP() {
		foreach (array('HTTP_X_FORWARDED','HTTP_VIA','REMOTE_ADDR') as $var) {
			if (isset($_SERVER[$var])) return $_SERVER[$var];
		}
		return 'unknown';
	}
}
?>