<?php
/*
	+-----------------------------------------------------------------------------+
	| GROUP-E collaboration software                                              |
	+-----------------------------------------------------------------------------+
	| Copyright (c) 2002-2005 group-e,                                            |
	|         Endo7 GmbH/Srl                                                      |
	|	  via-Goethe-strasse 34a                                              |
	|	  39100 Bozen/Bolzano                                                 |
	|	  ITALIEN/ITALIA                                                      |
	|         contact@endo7.com                                                   |
	| This program is free software; you can redistribute it and/or               |
	| modify it under the terms of the GNU General Public License                 |
	| as published by the Free Software Foundation; either version 2              |
	| of the License, or (at your option) any later version.                      |
	|                                                                             |
	| This program is distributed in the hope that it will be useful,             |
	| but WITHOUT ANY WARRANTY; without even the implied warranty of              |
	| MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               |
	| GNU General Public License for more details.                                |
	|                                                                             |
	| You should have received a copy of the GNU General Public License           |
	| along with this program; if not, write to the Free Software                 |
	| Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. |
        | http://www.fsf.org/                                                         |
	+-----------------------------------------------------------------------------+
*/
class groupeWebDavBackend extends ezcWebdavSimpleBackend implements ezcWebdavLockBackend {
    protected $_frm;
    protected $root;
    protected $lockLevel = 0;
    protected $lockFileName='.lock';
    protected $handledLiveProperties = array(
        'getcontentlength',
        'getlastmodified',
        'creationdate',
        'displayname',
        'getetag',
        'getcontenttype',
        'resourcetype',
        'supportedlock',
        'lockdiscovery',
    );
	protected $_DavFmng;
    public function __construct(Framework $frm)
    {
		$this->_frm=$frm;
		$root=GE_DYN_PATH.'/'.GE_DOMAIN_PATH;
        if ( !is_dir( $root ) )
        {
            throw new ezcBaseFileNotFoundException( $root );
        }
        if ( !is_readable( $root ) )
        {
            throw new ezcBaseFilePermissionException( $root, ezcBaseFileException::READ );
        }
        $this->root = realpath( $root ).'/';
        $this->_DavFmng=groupeWebDavFmng::getInstance();
    }
    /**
     * Returns a property of a resource.
     *
     * Returns the property with the given $propertyName, from the resource
     * identified by $path. You may optionally define a $namespace to receive
     * the property from.
     *
     * @param string $path
     * @param string $propertyName
     * @param string $namespace
     * @return ezcWebdavProperty
     */
    public function getProperty( $path, $propertyName, $namespace = 'DAV:' ) {
		$storage = $this->getPropertyStorage( $path );
		// Handle dead propreties
		if ( $namespace !== 'DAV:' )  {
			$properties = $storage->getAllProperties();
			return $properties[$namespace][$name];
		}
		debugFile('getProperty - '.$path.' - '.$propertyName,4);
		// Handle live properties
		switch ( $propertyName )
		{
			case 'getcontentlength':
				$property = new ezcWebdavGetContentLengthProperty();
				$property->length = $this->getContentLength( $path );
				return $property;

			case 'getlastmodified':
				$property = new ezcWebdavGetLastModifiedProperty();
				$property->date = new ezcWebdavDateTime( '@' . $this->_DavFmng->getMeta($path,'TimeModify') );
				return $property;
			case 'creationdate':
				$property = new ezcWebdavCreationDateProperty();
				$property->date = new ezcWebdavDateTime( '@' . $this->_DavFmng->getMeta($path,'TimeCreate') );
				return $property;
			case 'displayname':
				$property = new ezcWebdavDisplayNameProperty();
				$property->displayName = $this->_DavFmng->getName($path);
				return $property;
			case 'getcontenttype':
				$property = new ezcWebdavGetContentTypeProperty(
					$this->_DavFmng->getMimeType( $path )
				);
				return $property;
			case 'getetag':
				$property = new ezcWebdavGetEtagProperty();
				$property->etag = $this->getETag( $path );
				return $property;
			case 'resourcetype':
				$property = new ezcWebdavResourceTypeProperty();
				$property->type = $this->isCollection( $path ) ?
					ezcWebdavResourceTypeProperty::TYPE_COLLECTION :
					ezcWebdavResourceTypeProperty::TYPE_RESOURCE;
				return $property;
			case 'supportedlock':
				$property = new ezcWebdavSupportedLockProperty();
				return $property;

			case 'lockdiscovery':
				$property = $storage->get( 'lockdiscovery',$namespace );
				if (!$property) {
					$property = new ezcWebdavLockDiscoveryProperty();
				}
				//debugFile($property,4);
				return $property;

			default:
				$properties = $storage->getAllProperties();
				return $properties[$namespace][$name];
		}
    }

    /**
     * Returns all properties for a resource.
     *
     * Returns all properties for the resource identified by $path as a {@link
     * ezcWebdavBasicPropertyStorage}.
     *
     * @param string $path
     * @return ezcWebdavPropertyStorage
     */
    public function getAllProperties( $path )
    {
		debugFile('getAllProperties - '.$path,4);
        $storage = $this->getPropertyStorage( $path );
        // Add all live properties to stored properties
        foreach ( $this->handledLiveProperties as $property )
        {
            $storage->attach(
                $this->getProperty( $path, $property )
            );
        }
       return $storage;
    }
    /**
     * Manually sets a property on a resource.
     *
     * Sets the given $propertyBackup for the resource identified by $path.
     *
     * @param string $path
     * @param ezcWebdavProperty $property
     * @return bool
     */
    public function setProperty( $path, ezcWebdavProperty $property )  {
		debugFile('setProperty - '.$path.' - '.$property->namespace.$property->name,4);
        // Check if property is a self handled live property and return an
        // error in this case.
        if ( ( $property->namespace === 'DAV:' ) &&
             in_array( $property->name, $this->handledLiveProperties, true ) &&
             ( $property->name !== 'getcontenttype' ) &&
             ( $property->name !== 'lockdiscovery' ) ) {
            return false;
        }

        // Get namespace property storage
        $storage = $this->getPropertyStorage( $path );

        // Attach property to store
        $storage->attach( $property );

        // Store document back
        $this->storeProperties( $path, $storage );

        return true;
    }

    /**
     * Manually removes a property from a resource.
     *
     * Removes the given $property form the resource identified by $path.
     *
     * @param string $path
     * @param ezcWebdavProperty $property
     * @return bool
     */
    public function removeProperty( $path, ezcWebdavProperty $property ) {
		debugFile('removeProperty - '.$path.' - '.$property->namespace.$property->name,4);
        // Live properties may not be removed.
        if ( $property instanceof ezcWebdavLiveProperty )
        {
            return false;
        }

        // Get namespace property storage
        $storage = $this->getPropertyStorage( $path );

        // Attach property to store
        $storage->detach( $property->name, $property->namespace );

        // Store document back
        $this->storeProperties( $path, $storage );

        return true;
    }

    /**
     * Resets the property storage for a resource.
     *
     * Discardes the current {@link ezcWebdavPropertyStorage} of the resource
     * identified by $path and replaces it with the given $properties.
     *
     * @param string $path
     * @param ezcWebdavPropertyStorage $storage
     * @return bool
     */
    public function resetProperties( $path, ezcWebdavPropertyStorage $storage )
    {
	   debugFile('resetProperties - '.$path,4);
       $this->storeProperties( $path, $storage );
    }

     /**
     * Returns the property storage for a resource.
     *
     * Returns the {@link ezcWebdavPropertyStorage} instance containing the
     * properties for the resource identified by $path.
     *
     * @param string $path
     * @return ezcWebdavBasicPropertyStorage
     */
    protected function getPropertyStorage( $path ) {
		debugFile('getPropertyStorage:'.$path,4);
		$davXml=$this->_DavFmng->getMeta($path,'DavProperties');
		if (!trim($davXml)) {
			return new ezcWebdavBasicPropertyStorage();
		}
        // Create handler structure to read properties
        $handler = new ezcWebdavPropertyHandler(
            $xml = new ezcWebdavXmlTool()
        );
        $storage = new ezcWebdavBasicPropertyStorage();
        // Read document
        try
        {
             $doc = $xml->createDom($davXml);
        }
        catch ( ezcWebdavInvalidXmlException $e )
        {
            throw new ezcWebdavFileBackendBrokenStorageException(
                "Could not open XML as DOMDocument: '{$storage}'."
            );
        }
        // Get property node from document
        $properties = $doc->getElementsByTagname( 'properties' )->item( 0 )->childNodes;
        // Extract and return properties
        $handler->extractProperties(
            $properties,
            $storage
        );
		//debugFile($storage);
        return $storage;
    }

    protected function storeProperties( $path, ezcWebdavBasicPropertyStorage $storage )   {
		debugFile('storeProperties:'.$path,4);
        // Create handler structure to read properties
        $handler = new ezcWebdavPropertyHandler(
            $xml = new ezcWebdavXmlTool()
        );

        // Create new dom document with property storage for one namespace
        $doc = new DOMDocument( '1.0' );

        $properties = $doc->createElement( 'properties' );
        $doc->appendChild( $properties );
        // Store and store properties
        $handler->serializeProperties(
            $storage,
            $properties
        );
		return $this->_DavFmng->storeProperties($path, $doc->saveXML());
    }

    private function getContentLength( $path )  {
        $length = ezcWebdavGetContentLengthProperty::COLLECTION;
        if ( !$this->isCollection( $path ) )  {
            $length = (string) $this->_DavFmng->getFileMeta($path,'FileSize');
        }
        return $length;
    }

    /**
     * Returns the mime type of a resource.
     *
     * Return the mime type of the resource identified by $path. If a mime type
     * extension is available it will be used to read the real mime type,
     * otherwise the original mime type passed by the client when uploading the
     * file will be returned. If no mimetype has ever been associated with the
     * file, the method will just return 'application/octet-stream'.
     *
     * @param string $path
     * @return string
     */

    /**
     * Returns the etag representing the current state of $path.
     *
     * Calculates and returns the ETag for the resource represented by $path.
     * The ETag is calculated from the $path itself and the following
     * properties, which are concatenated and md5 hashed:
     *
     * <ul>
     *  <li>getcontentlength</li>
     *  <li>getlastmodified</li>
     * </ul>
     *
     * This method can be overwritten in custom backend implementations to
     * access the information needed directly without using the way around
     * properties.
     *
     * Custom backend implementations are encouraged to use the same mechanism
     * (or this method itself) to determine and generate ETags.
     *
     * @param mixed $path
     * @return void
     */

    protected function getETag( $path )
    {
		return md5(
			$path
			. $this->getContentLength( $path )
			. date( 'c',  $this->_DavFmng->getMeta($path,'TimeCreate'))
		);
    }

    /**
     * Returns if resource is a collection.
     *
     * Returns if the resource identified by $path is a collection resource
     * (true) or a non-collection one (false).
     *
     * @param string $path
     * @return bool
     */
    protected function isCollection( $path ) {
		return  $this->_DavFmng->isFolder($path);
    }

   /**
     * Create a new collection.
     *
     * Creates a new collection at the given path.
     *
     * @param string $path
     * @return void
     */
    protected function createCollection( $path ) {
		debugFile('createCollection:'.$path,1);
		$this->_DavFmng->createFolder($path);

    }

    /**
     * Create a new resource.
     *
     * Creates a new resource at the given path, optionally with the given
     * content.
     *
     * @param string $path
     * @param string $content
     * @return void
     */
    protected function createResource( $path, $content = null )   {
		debugFile('createResource:'.$path,1);
		$this->_DavFmng->createFile($path,$content);
    }

    /**
     * Set contents of a resource.
     *
     * Change the contents of the given resource to the given content.
     *
     * @param string $path
     * @param string $content
     * @return void
     */
    protected function setResourceContents( $path, $content ) {
		debugFile('setResourceContents:'.$path,1);
		$this->_DavFmng->replaceContent( $path, $content );
    }

    /**
     * Get contents of a resource.
     *
     * @param string $path
     * @return string
     */
    protected function getResourceContents( $path )  {
		debugFile('getResourceContents:'.$path,1);
		if (file_exists($this->_DavFmng->getFileMeta($path,'FilePath'))) {
			return file_get_contents($this->_DavFmng->getFileMeta($path,'FilePath'));
		}
    }


   /**
     * Returns members of collection.
     *
     * Returns an array with the members of the collection identified by $path.
     * The returned array can contain {@link ezcWebdavCollection}, and {@link
     * ezcWebdavResource} instances and might also be empty, if the collection
     * has no members.
     *
     * @param string $path
     * @return array(ezcWebdavResource|ezcWebdavCollection)
     */
    protected function getCollectionMembers($path) {
		debugFile('getCollectionMembers:'.$path,1);
		$contents=$this->_DavFmng->getSubEntries($path);
		//debugFile($contents,1);
		return $contents;
    }


    /**
     * Copies resources recursively from one path to another.
     *
     * Copies the resourced identified by $fromPath recursively to $toPath with
     * the given $depth, where $depth is one of {@link
     * ezcWebdavRequest::DEPTH_ZERO}, {@link ezcWebdavRequest::DEPTH_ONE},
     * {@link ezcWebdavRequest::DEPTH_INFINITY}.
     *
     * Returns an array with {@link ezcWebdavErrorResponse}s for all subtrees,
     * where the copy operation failed. Errors for subsequent resources in a
     * subtree should be ommitted.
     *
     * If an empty array is return, the operation has been completed
     * successfully.
     *
     * @param string $fromPath
     * @param string $toPath
     * @param int $depth
     * @return array(ezcWebdavErrorResponse)
     */
    protected function performCopy( $fromPath, $toPath, $depth = ezcWebdavRequest::DEPTH_INFINITY ) {
		debugFile('performCopy:'.$fromPath.'->'.$toPath,1);
		$errors=$this->_DavFmng->copyObj($fromPath,$toPath,$depth);
		if (is_array($errors)) {
			return $errors;
		} else {
			return array();
		}
    }

    protected function performMove( $fromPath, $toPath, $depth = ezcWebdavRequest::DEPTH_INFINITY ) {
		debugFile('performMove:'.$fromPath.'->'.$toPath,1);
		$errors=$this->_DavFmng->moveObj($fromPath,$toPath);
		if ($errors) {
			return new ezcWebdavMultistatusResponse(
					$errors
			);
		} else {
			return null;
		}
   }
   /**
     * Deletes everything below a path.
     *
     * Deletes the resource identified by $path recursively. Returns an
     * instance of {@link ezcWebdavErrorResponse} if the deletion failed, and
     * null on success.
     *
     * @param string $path
     * @return ezcWebdavErrorResponse
     */
    protected function performDelete( $path ) {
		debugFile('performDelete:'.$path,1);
		$errors=$this->_DavFmng->deleteObj($path);
		if ($errors) {
			return new ezcWebdavMultistatusResponse(
					$errors
			);
		} else {
			return null;
		}
    }

    /**
     * Returns if a resource exists.
     *
     * Returns if a the resource identified by $path exists.
     *
     * @param string $path
     * @return bool
     */
    protected function nodeExists( $path ) {
		$result=$this->_DavFmng->exists($path);
		debugFile('PathExist:'.$path.':'.$result,1);
        return $result;
    }

     /**
     * Serves GET requests.
     *
     * The method receives a {@link ezcWebdavGetRequest} object containing all
     * relevant information obout the clients request and will return an {@link
     * ezcWebdavErrorResponse} instance on error or an instance of {@link
     * ezcWebdavGetResourceResponse} or {@link ezcWebdavGetCollectionResponse}
     * on success, depending on the type of resource that is referenced by the
     * request.
     *
     * This method acquires the internal lock of the backend, dispatches to
     * {@link ezcWebdavSimpleBackend} to perform the operation and releases the
     * lock afterwards.
     *
     * @param ezcWebdavGetRequest $request
     * @return ezcWebdavResponse
     */
   public function get( ezcWebdavGetRequest $request )
    {
		//debug($request);
        $this->acquireLock( true );
        $return = parent::get( $request );
        $this->freeLock();
        return $return;
    }
    /**
     * Serves HEAD requests.
     *
     * The method receives a {@link ezcWebdavHeadRequest} object containing all
     * relevant information obout the clients request and will return an {@link
     * ezcWebdavErrorResponse} instance on error or an instance of {@link
     * ezcWebdavHeadResponse} on success.
     *
     * This method acquires the internal lock of the backend, dispatches to
     * {@link ezcWebdavSimpleBackend} to perform the operation and releases the
     * lock afterwards.
     *
     * @param ezcWebdavHeadRequest $request
     * @return ezcWebdavResponse
     */
    public function head( ezcWebdavHeadRequest $request )
    {
        $this->acquireLock( true );
        $return = parent::head( $request );
        $this->freeLock();

        return $return;
    }
    /**
     * Serves PROPFIND requests.
     *
     * The method receives a {@link ezcWebdavPropFindRequest} object containing
     * all relevant information obout the clients request and will either
     * return an instance of {@link ezcWebdavErrorResponse} to indicate an error
     * or a {@link ezcWebdavPropFindResponse} on success. If the referenced
     * resource is a collection or if some properties produced errors, an
     * instance of {@link ezcWebdavMultistatusResponse} may be returned.
     *
     * The {@link ezcWebdavPropFindRequest} object contains a definition to
     * find one or more properties of a given collection or non-collection
     * resource.
     *
     * This method acquires the internal lock of the backend, dispatches to
     * {@link ezcWebdavSimpleBackend} to perform the operation and releases the
     * lock afterwards.
     *
     * @param ezcWebdavPropFindRequest $request
     * @return ezcWebdavResponse
     */
    public function propFind( ezcWebdavPropFindRequest $request )
    {
        $this->acquireLock( true );
       // debugFile('propFind'.print_r($request,1),1);
        $return = parent::propFind( $request );
        //debugFile('propFindResult'.print_r($return,1),1);
        $this->freeLock();
        return $return;
    }
    /**
     * Serves PROPPATCH requests.
     *
     * The method receives a {@link ezcWebdavPropPatchRequest} object
     * containing all relevant information obout the clients request and will
     * return an instance of {@link ezcWebdavErrorResponse} on error or a
     * {@link ezcWebdavPropPatchResponse} response on success. If the
     * referenced resource is a collection or if only some properties produced
     * errors, an instance of {@link ezcWebdavMultistatusResponse} may be
     * returned.
     *
     * This method acquires the internal lock of the backend, dispatches to
     * {@link ezcWebdavSimpleBackend} to perform the operation and releases the
     * lock afterwards.
     *
     * @param ezcWebdavPropPatchRequest $request
     * @return ezcWebdavResponse
     */
    public function propPatch( ezcWebdavPropPatchRequest $request )
    {
        $this->acquireLock();
        $return = parent::propPatch( $request );
        $this->freeLock();
        return $return;
    }
   /**
     * Serves PUT requests.
     *
     * The method receives a {@link ezcWebdavPutRequest} objects containing all
     * relevant information obout the clients request and will return an
     * instance of {@link ezcWebdavErrorResponse} on error or {@link
     * ezcWebdavPutResponse} on success.
     *
     * @param ezcWebdavPutRequest $request
     * @return ezcWebdavResponse
     */
    public function put( ezcWebdavPutRequest $request ) {
        $this->acquireLock();
        $return = $this->_put( $request );
        $this->freeLock();
        return $return;
    }
    private function _put( ezcWebdavPutRequest $request ) {
        $source = $request->requestUri;
        // Check authorization
        // Need to do this before checking of node existence is checked, to
        // avoid leaking information
        if ( !ezcWebdavServer::getInstance()->isAuthorized( $source, $request->getHeader( 'Authorization' ), ezcWebdavAuthorizer::ACCESS_WRITE ) )
        {
            return $this->createUnauthorizedResponse(
                $source,
                $request->getHeader( 'Authorization' )
            );
        }

        // Check if parent node exists and throw a 409 otherwise
        if ( !$this->nodeExists( dirname( $source ) ) )
        {
            return new ezcWebdavErrorResponse(
                ezcWebdavResponse::STATUS_409,
                $source
            );
        }

        // Check if parent node is a collection, and throw a 409 otherwise
        if ( !$this->isCollection( dirname( $source ) ) )
        {
            return new ezcWebdavErrorResponse(
                ezcWebdavResponse::STATUS_409,
                $source
            );
        }

        // Check if resource to be updated or created does not exists already
        // AND is a collection
        if ( $this->nodeExists( $source ) && $this->isCollection( $source ) )
        {
            return new ezcWebdavErrorResponse(
                ezcWebdavResponse::STATUS_409,
                $source
            );
        }

        // @todo: RFC2616 Section 9.6 PUT requires us to send 501 on all
        // Content-* we don't support.

        // Verify If-[None-]Match headers
        if ( $this->nodeExists( $source ) && ( $res = $this->checkIfMatchHeaders( $request, $source ) ) !== null )
        {
            return $res;
        }

        // Everything is OK, create or update resource.
        if ( !$this->nodeExists( $source ) )
        {
            $this->createResource( $source );
        }
        $this->setResourceContents( $source, $request->body );

        $res = new ezcWebdavPutResponse(
            $source
        );

        // Add ETag header
        $res->setHeader( 'ETag', $this->getETag( $source ) );
        return $res;
    }

    /**
     * Serves MOVE requests.
     *
     * The method receives a {@link ezcWebdavMoveRequest} objects containing
     * all relevant information obout the clients request and will return an
     * instance of {@link ezcWebdavErrorResponse} on error or {@link
     * ezcWebdavMoveResponse} on success. If only some operations failed, this
     * method may return an instance of {@link ezcWebdavMultistatusResponse}.
     *
     * @param ezcWebdavMoveRequest $request
     * @return ezcWebdavResponse
     */
    public function move( ezcWebdavMoveRequest $request ) {
        $this->acquireLock();
        $return = $this->_move( $request );
        $this->freeLock();
        return $return;
    }

    private function _move( ezcWebdavMoveRequest $request )  {
        // Indicates wheather a destiantion resource has been replaced or not.
        // The success response code depends on this.
        $replaced = false;

        // Extract paths from request
        $source = $request->requestUri;
        $dest = $request->getHeader( 'Destination' );

        if ($this->isCollection($source)) {
			$accessCheck=groupeWebDavAuthorizer::ACCESS_MANAGE;
        } else {
			$accessCheck=groupeWebDavAuthorizer::ACCESS_WRITE;
        }

        // Check authorization
        // Need to do this before checking of node existence is checked, to
        // avoid leaking information
        $authState = $this->recursiveAuthCheck(
            $request,
            $source,  // geändert in Source
            $accessCheck,
            true
        );
        if ( count( $authState['errors'] ) !== 0 )
        {
            // Source permission denied
            return $authState['errors'][0];
        }
        
        if ( !ezcWebdavServer::getInstance()->isAuthorized( $dest, $request->getHeader( 'Authorization' ), groupeWebDavAuthorizer::ACCESS_WRITE ) )
        {
            return $this->createUnauthorizedResponse(
                $dest,
                $request->getHeader( 'Authorization' )
            );
        }

        // Check if resource is available
        if ( !$this->nodeExists( $source ) )
        {
            return new ezcWebdavErrorResponse(
                ezcWebdavResponse::STATUS_404,
                $source
            );
        }

        // If source and destination are equal, the request should always fail.
        if ( $source === $dest )
        {
            return new ezcWebdavErrorResponse(
                ezcWebdavResponse::STATUS_403,
                $source
            );
        }

        // Check if destination resource exists and throw error, when
        // overwrite header is F
        if ( ( $request->getHeader( 'Overwrite' ) === 'F' ) &&
             $this->nodeExists( $dest ) )
        {
            return new ezcWebdavErrorResponse(
                ezcWebdavResponse::STATUS_412,
                $dest
            );
        }

        // Check if the destination parent directory already exists, otherwise
        // bail out.
        if ( !$this->nodeExists( $destDir = dirname( $dest ) ) )
        {
            return new ezcWebdavErrorResponse(
                ezcWebdavResponse::STATUS_409,
                $dest
            );
        }

        // Verify If-[None-]Match headers on the $source
        $res = $this->checkIfMatchHeadersRecursive(
            $request,
            $source,
            // We move, not copy!
            ezcWebdavRequest::DEPTH_INFINITY
        );
        if ( $res !== null )
        {
            return $res;
        }

        // Verify If-[None-]Match headers on the $dest if it exists
        if ( $this->nodeExists( $dest ) &&
             ( $res = $this->checkIfMatchHeaders( $request, $dest ) ) !== null
           )
        {
            return $res;
        }
        // Verify If-[None-]Match headers on the on $dests parent dir, if it
        // does not exist
        elseif ( ( $res = $this->checkIfMatchHeaders( $request, $destDir ) ) !== null )
        {
            return $res;
        }

        // The destination resource should be deleted if it exists and the
        // overwrite headers is T
        if ( ( $request->getHeader( 'Overwrite' ) === 'T' ) &&
             $this->nodeExists( $dest ) )
        {
            // Check sub-sequent authorization on destination
            $authState = $this->recursiveAuthCheck(
                $request,
                $dest,
                groupeWebDavAuthorizer::ACCESS_MANAGE,
                true
            );
            if ( count( $authState['errors'] ) !== 0 )
            {
                // Permission denied on deleting destination
                return $authState['errors'][0];
            }

            $replaced = true;

            if ( count( $delteErrors = $this->performDelete( $dest ) ) > 0 )
            {
                return new ezcWebdavMultistatusResponse( $delteErrors );
            }
        }

        // All checks are passed, we can actuall copy now.
        //
        // MOVEd contents should always be copied using infinity depth.
        //
        // @todo: handle keepalive setting somehow - even the RFC is quite
        // vague how to handle them exactly.
        $errors = $this->performMove( $source, $dest, ezcWebdavRequest::DEPTH_INFINITY );

        // If an error occured we skip deletion of source.
        //
        // @IMPORTANT: This is a definition / assumption made by us, because it
        // is not defined in the RFC how to handle such a case.
        if ( count( $errors ) )
        {
            // We need a multistatus response, because some errors occured for some
            // of the resources.
            return new ezcWebdavMultistatusResponse( $errors );
        }

        // Send proper response on success
        return new ezcWebdavMoveResponse(
            $replaced
        );
    }

    /**
     * Serves DELETE requests.
     *
     * The method receives a {@link ezcWebdavDeleteRequest} objects containing
     * all relevant information obout the clients request and will return an
     * instance of {@link ezcWebdavErrorResponse} on error or {@link
     * ezcWebdavDeleteResponse} on success.
     *
     * @param ezcWebdavDeleteRequest $request
     * @return ezcWebdavResponse
     */
    public function delete( ezcWebdavDeleteRequest $request ) {
        $this->acquireLock();
        $return = $this->_delete( $request );
        $this->freeLock();
        return $return;
    }
    private function _delete( ezcWebdavDeleteRequest $request ) {
        $source = $request->requestUri;
        // Check authorization
        // Need to do this before checking of node existence is checked, to
        // avoid leaking information
        $authState = $this->recursiveAuthCheck(
            $request,
            $source,
            groupeWebDavAuthorizer::ACCESS_MANAGE,
            true
        );
        if ( count( $authState['errors'] ) !== 0 )
        {
            return $authState['errors'][0];
        }

        // Check if resource is available
        if ( !$this->nodeExists( $source ) )
        {
            return new ezcWebdavErrorResponse(
                ezcWebdavResponse::STATUS_404,
                $source
            );
        }

        // Verify If-[None-]Match headers
        // @todo: Does this make sense for PROPFIND requests?
        $res = $this->checkIfMatchHeadersRecursive(
            $request,
            $source,
            ezcWebdavRequest::DEPTH_INFINITY
        );
        if ( $res !== null )
        {
            return $res;
        }

        // Delete
        $deletion = $this->performDelete( $source );
        if ( $deletion !== null )
        {
            // ezcWebdavMultistatusResponse
            return $deletion;
        }
        // Send proper response on success
        return new ezcWebdavDeleteResponse(
            $source
        );
    }

    /**
     * Serves COPY requests.
     *
     * The method receives a {@link ezcWebdavCopyRequest} objects containing
     * all relevant information obout the clients request and will return an
     * instance of {@link ezcWebdavErrorResponse} on error or {@link
     * ezcWebdavCopyResponse} on success. If only some operations failed, this
     * method may return an instance of {@link ezcWebdavMultistatusResponse}.
     *
     * @param ezcWebdavCopyRequest $request
     * @return ezcWebdavResponse
     */
    public function copy( ezcWebdavCopyRequest $request ) {
        $this->acquireLock();
        $return = $this->_copy( $request );
        $this->freeLock();
        return $return;
    }
    private function _copy( ezcWebdavCopyRequest $request )  {
        // Indicates wheather a destiantion resource has been replaced or not.
        // The success response code depends on this.
        $replaced = false;

        // Extract paths from request
        $source = $request->requestUri;
        $dest = $request->getHeader( 'Destination' );

        if ($this->isCollection($source)) {
			$accessCheck=groupeWebDavAuthorizer::ACCESS_MANAGE;
        } else {
			$accessCheck=groupeWebDavAuthorizer::ACCESS_WRITE;
        }
      // Check authorization
        // Need to do this before checking of node existence is checked, to
        // avoid leaking information
        if ( !ezcWebdavServer::getInstance()->isAuthorized( $source, $request->getHeader( 'Authorization' ) ) )
        {
            return $this->createUnauthorizedResponse(
                $source,
                $request->getHeader( 'Authorization' )
            );
        }
        if ( !ezcWebdavServer::getInstance()->isAuthorized( $dest, $request->getHeader( 'Authorization' ), $accessCheck ) )
        {
            return $this->createUnauthorizedResponse(
                $dest,
                $request->getHeader( 'Authorization' )
            );
        }

        // Check if resource is available
        if ( !$this->nodeExists( $source ) )
        {
            return new ezcWebdavErrorResponse(
                ezcWebdavResponse::STATUS_404,
                $source
            );
        }

        // If source and destination are equal, the request should always fail.
        if ( $source === $dest )
        {
            return new ezcWebdavErrorResponse(
                ezcWebdavResponse::STATUS_403,
                $source
            );
        }

        // Check if destination resource exists and throw error, when
        // overwrite header is F
        if ( ( $request->getHeader( 'Overwrite' ) === 'F' ) &&
             $this->nodeExists( $dest ) )
        {
            return new ezcWebdavErrorResponse(
                ezcWebdavResponse::STATUS_412,
                $dest
            );
        }

        // Check if the destination parent directory already exists, otherwise
        // bail out.
        if ( !$this->nodeExists( $destDir = dirname( $dest ) ) )
        {
            return new ezcWebdavErrorResponse(
                ezcWebdavResponse::STATUS_409,
                $dest
            );
        }

        // Verify If-[None-]Match headers on the $source
        $res = $this->checkIfMatchHeadersRecursive(
            $request,
            $source,
            $request->getHeader( 'Depth' )
        );
        if ( $res !== null )
        {
            return $res;
        }

        // Verify If-[None-]Match headers on the $dest if it exists
        if ( $this->nodeExists( $dest ) &&
             ( $res = $this->checkIfMatchHeaders( $request, $dest ) ) !== null
           )
        {
            return $res;
        }
        // Verify If-[None-]Match headers on the on $dests parent dir, if it
        // does not exist
        elseif ( ( $res = $this->checkIfMatchHeaders( $request, $destDir ) ) !== null )
        {
            return $res;
        }

        // The destination resource should be deleted if it exists and the
        // overwrite headers is T
        if ( ( $request->getHeader( 'Overwrite' ) === 'T' ) &&
             $this->nodeExists( $dest ) )
        {
            // Check sub-sequent authorization on destination
            $authState = $this->recursiveAuthCheck(
                $request,
                $dest,
                groupeWebDavAuthorizer::ACCESS_MANAGE,
                true
            );
            if ( count( $authState['errors'] ) !== 0 )
            {
                // Permission denied on deleting destination
                return $authState['errors'][0];
            }

            // Perform delete
            // @todo: This method might return errors. If it does, the delete
            // was not successful and therefore no copy should happen! (see:
            // move()).
            $replaced = true;
            $this->performDelete( $dest );
        }

        $errors    = array();
        $copyPaths = array();

        if ( $request->getHeader( 'Depth' ) === ezcWebdavRequest::DEPTH_INFINITY )
        {
            $authState = $this->recursiveAuthCheck( $request, $source );
            $errors    = $authState['errors'];
            $copyPaths = $authState['paths'];
        }
        else
        {
            // Non recursive auth check necessary, plain check on $source
            // already performed
            $copyPaths = array( $source => ezcWebdavRequest::DEPTH_ZERO );
        }

        // Recursively copy paths that should be copied
        foreach ( $copyPaths as $copySource => $copyDepth )
        {
            // Build destination path fur descendants
            $copyDest = $dest . (string) substr( $copySource, strlen( $source ) );
            // Perform copy and collect additional errors.
            $errors = array_merge(
                $errors,
                // @todo: handle keepalive setting somehow - even the RFC is quite
                // vague how to handle them exactly.
                $this->performCopy( $copySource, $copyDest, $copyDepth )
            );
        }

        if ( !count( $errors ) )
        {
            // No errors occured during copy. Just response with success.
            return new ezcWebdavCopyResponse(
                $replaced
            );
        }
        // Send proper response on success
        return new ezcWebdavMultistatusResponse( $errors );
    }


    /**
     * Serves MKCOL (make collection) requests.
     *
     * The method receives a {@link ezcWebdavMakeCollectionRequest} objects
     * containing all relevant information obout the clients request and will
     * return an instance of {@link ezcWebdavErrorResponse} on error or {@link
     * ezcWebdavMakeCollectionResponse} on success.
     *
     * @param ezcWebdavMakeCollectionRequest $request
     * @return ezcWebdavResponse
     */
    public function makeCollection( ezcWebdavMakeCollectionRequest $request ) {
        $this->acquireLock();
        $return = $this->_makeCollection( $request );
        $this->freeLock();
        return $return;
    }
    private function _makeCollection( ezcWebdavMakeCollectionRequest $request ) {
        $collection = $request->requestUri;
        // Check authorization
        // Need to do this before checking of node existence is checked, to
        // avoid leaking information
        if ( !ezcWebdavServer::getInstance()->isAuthorized( $collection, $request->getHeader( 'Authorization' ), groupeWebDavAuthorizer::ACCESS_MANAGE ) )
        {
            return $this->createUnauthorizedResponse(
                $collection,
                $request->getHeader( 'Authorization' )
            );
        }

        // If resource already exists, the collection cannot be created and a
        // 405 is thrown.
        if ( $this->nodeExists( $collection ) )
        {
            return new ezcWebdavErrorResponse(
                ezcWebdavResponse::STATUS_405,
                $collection
            );
        }

        // Check if the parent node already exists, otherwise throw a 409
        // error.
        if ( !$this->nodeExists( dirname( $collection ) ) )
        {
            return new ezcWebdavErrorResponse(
                ezcWebdavResponse::STATUS_409,
                $collection
            );
        }

        // If the parent node exists, but is a resource, which obviously can
        // not accept any members, throw a 403 error.
        if ( !$this->isCollection( $destDir = dirname( $collection ) ) )
        {
            return new ezcWebdavErrorResponse(
                ezcWebdavResponse::STATUS_403,
                $collection
            );
        }

        // Verify If-[None-]Match headers on the on $dests parent dir
        if ( ( $res = $this->checkIfMatchHeaders( $request, $destDir ) ) !== null )
        {
            return $res;
        }

        // As the handling of request bodies is not described in RFC 2518, we
        // skip their handling and always return a 415 error.
        if ( $request->body )
        {
            return new ezcWebdavErrorResponse(
                ezcWebdavResponse::STATUS_415,
                $collection
            );
        }

        // Cause error, if requested?
        // All checks passed, we can create the collection
        $this->createCollection( $collection );
         // Return success
        return new ezcWebdavMakeCollectionResponse(
            $collection
        );
    }
    private function createUnauthorizedResponse( $path, $authHeader = null )  {
        // Check for anonymous auth
        if ( $authHeader === null || $authHeader->username === '' )
        {
            return ezcWebdavServer::getInstance()->createUnauthenticatedResponse(
                $path,
                'Authorization failed.'
            );
        }

        // Authenticated user does not have access
        return ezcWebdavServer::getInstance()->createUnauthorizedResponse(
            $path,
            'Authorization failed.'
        );
    }
    /**
     * Recursively checks authorization for the COPY, MOVE and other requests.
     *
     * This method performs a recursive authorization check on the given $path
     * using the credentials provided in $request. It returns a
     * multidimensional array, indicating the authorization errors occurred and
     * the paths that may by copied.
     *
     * The structure looks like this:
     * <code>
     * array(
     *      'errors' => array(
     *          ezcWebdavErrorResponse(),
     *          ezcWebdavErrorResponse(),
     *          // ...
     *      )
     *      'paths' => array(
     *          '/some/path' => ezcWebdavRequest::DEPTH_INFINITY,
     *          '/some/other/path' => ezcWebdavRequest::DEPTH_ZERO,
     *          // ...
     *      )
     * )
     * </code>
     *
     * The 'errors' key is assigned to an array of authorization error
     * responses that will be merged to the ezcWebdavMultistatusResponse
     * returned by the copy() method. The 'paths' array contains all paths that
     * may be copied by the method. A path is assigned to the depth that it
     * might be copied. The depth can be {@link
     * ezcWebdavRequest::DEPTH_INFINITY} to indicate that a complete sub tree
     * is save for copying, or {@link ezcWebdavRequest::DEPTH_ZERO}, to
     * indicate that only the path itself may be copied, but none of its
     * descendants.
     *
     * The $access parameter specifies which permission is to be checked {@link
     * ezcWebdavAuthorizer::ACCESS_READ} is the default, {@link
     * ezcWebdavAuthorizer::ACCESS_WRITE} may be set to indicate write
     * permissions.
     *
     * If the $breakOnError parameter is set to true, no further checks will be
     * applied to sibling resources, but the method will instantly return. This
     * parameter is set to true for the MOVE request, since this request must
     * be processed completly or not at all. The COPY request in contrast may
     * also be processed partially, so this parameter is left as is.
     *
     * @param ezcWebdavRequest $request
     * @param string $path
     * @param int $access
     * @param bool $breakOnError
     * @return array
     *
     * @todo Mark protected as soon as API is final.
     */
    private function recursiveAuthCheck( ezcWebdavRequest $request, $path, $access = ezcWebdavAuthorizer::ACCESS_WRITE, $breakOnError = false )
    {
        $result = array(
            'errors' => array(),
            'paths' => array(),
        );

        // Check auth for collections and resources equally
        if ( !ezcWebdavServer::getInstance()->isAuthorized( $path, $request->getHeader( 'Authorization' ), $access ) )
        {
            $result['errors'][] = $this->createUnauthorizedResponse(
                $path,
                $request->getHeader( 'Authorization' )
            );
        }
        else
        {
            if ( $this->isCollection( $path ) )
            {
                foreach ( $this->getCollectionMembers( $path ) as $member )
                {
                    $tmpRes = $this->recursiveAuthCheck( $request, $member->path, $access );
                    if ( count( $tmpRes['errors'] ) !== 0 )
                    {
                        if ( $breakOnError )
                        {
                            return $tmpRes;
                        }
                        $result['errors'] = array_merge( $result['errors'], $tmpRes['errors'] );
                        $result['paths']  = array_merge( $result['paths'],  $tmpRes['paths'] );
                    }
                }
                $result['paths'][$path] = ( count( $result['errors'] ) ? ezcWebdavRequest::DEPTH_ZERO : ezcWebdavRequest::DEPTH_INFINITY );
            }
            else
            {
                // Only a resource, so depth infinity does not make sense
                $result['paths'][$path] = ezcWebdavRequest::DEPTH_ZERO;
            }
        }

        return $result;
    }
    /**
     * Checks If-[Match-]Headers recursively on $path with $depth.
     *
     * Performs a recursive check using {@link checkIfMatchHeaders()} on $path.
     * $depth can be any of the {@link ezcWebdavRequest} DEPTH_* constants.
     *
     * Returns {@link ezcWebdavErrorResponse} if any ETag check failed, null if
     * everything went allright.
     *
     * @param ezcWebdavRequest $req
     * @param string $path
     * @param int $depth
     * @return ezcWebdavErrorResponse|null
     */
    private function checkIfMatchHeadersRecursive( ezcWebdavRequest $req, $path, $depth )
    {
        // Stop checking if non-collection is reached or depth is 0.
        if ( !$this->isCollection( $path ) || $depth === ezcWebdavRequest::DEPTH_ZERO )
        {
            return $this->checkIfMatchHeaders( $req, $path );
        }

        // Check collection ETag
        if ( ( $res = $this->checkIfMatchHeaders( $req, $path ) ) !== null )
        {
            return $res;
        }

        // $path is a collection, depth is > 0: Recurse.
        $newDepth = $depth === ezcWebdavRequest::DEPTH_ONE
            ? ezcWebdavRequest::DEPTH_ZERO
            : ezcWebdavRequest::DEPTH_INFINITY;

        foreach ( $this->getCollectionMembers( $path ) as $member )
        {
            // If-[None-]Match header check produced error.
            if ( ( $res = $this->checkIfMatchHeadersRecursive( $req, $member->path, $newDepth ) ) !== null )
            {
                return $res;
            }
        }
        return null;
    }
    /**
     * Checks the If-Match-Header on $path, if present in $req.
     *
     * Returns ezcWebdavErrorResponse on failure, otherwise null.
     *
     * @param ezcWebdavRequest $req
     * @param string $path
     * @return ezcWebdavErrorResponse|null
     */
    private function checkIfMatchHeader( ezcWebdavRequest $req, $path )
    {
        if ( ( $matches = $req->getHeader( 'If-Match' ) ) !== null )
        {
            $etag = $this->getETag( $path );
            if ( $this->checkMatchHeader( $matches, $etag ) === false )
            {
                return new ezcWebdavErrorResponse(
                    ezcWebdavResponse::STATUS_412,
                    $path,
                    'If-Match header check failed.'
                );
            }
        }
        return null;
    }

    /**
     * Checks the If-Match-Header on $path, if present in $req.
     *
     * Returns ezcWebdavErrorResponse on failure, otherwise null.
     *
     * @param ezcWebdavRequest $req
     * @param string $path
     * @return ezcWebdavErrorResponse|null
     */
    private function checkIfNoneMatchHeader( ezcWebdavRequest $req, $path )
    {
        if ( ( $matches = $req->getHeader( 'If-None-Match' ) ) !== null )
        {
            $etag = $this->getETag( $path );
            if ( $this->checkMatchHeader( $matches, $etag ) === true )
            {
                return new ezcWebdavErrorResponse(
                    ezcWebdavResponse::STATUS_412,
                    $path,
                    'If-None-Match header check failed.'
                );
            }
        }
        return null;
    }

    /**
     * Checks the If-[None-]Match header values against an $etag.
     *
     * Returns in any of the ETags given in $matches euqlauls to $etag.
     *
     * This is used in {@link checkIfMatchHeader()} and {@link
     * checkIfNoneMatchHeader()}. The {@link checkIfMatchHeader()} method
     * expects true as a good result, while {@link checkIfNoneMatchHeader()}
     * desires false.
     *
     * @param array(string) $matches
     * @param string $etag
     * @return bool
     */
    private function checkMatchHeader( $matches, $etag )
    {
        if ( $matches === true )
        {
            return true;
        }
        foreach ( $matches as $testEtag )
        {
            if ( $etag === $testEtag )
            {
                return true;
            }
        }
        return false;
    }

    /**
     * Locks the backend.
     *
     * Tries to lock the backend. If the lock is already owned by this process,
     * locking is successful. If $timeout is reached before a lock could be
     * acquired, an {@link ezcWebdavLockTimeoutException} is thrown. Waits
     * $waitTime microseconds between attempts to lock the backend.
     *
     * @param int $waitTime
     * @param int $timeout
     * @return void
     */
    public function lock( $waitTime, $timeout )
    {
		debugFile('lock:'.$waitTime.':'.$timeout,1);
        // Check and raise lockLevel counter
        if ( $this->lockLevel > 0 )
        {
            // Lock already acquired
            ++$this->lockLevel;
            return;
        }

        $lockStart = microtime( true );

        $lockFileName = $this->root . '/' . $this->lockFileName;

        // fopen in mode 'x' will only open the file, if it does not exist yet.
        // Even this is is expected it will throw a warning, if the file
        // exists, which we need to silence using the @
        while ( ( $fp = @fopen( $lockFileName, 'x' ) ) === false )
        {
 			debugFile('microtime:'.(microtime( true )-$lockStart).'>'.($timeout/1000000));
           // This is untestable.
            if ( microtime( true ) - $lockStart > ($timeout/1000000) )
            {
                // Release timed out lock
                unlink( $lockFileName );
                $lockStart = microtime( true );
            }
            else
            {
				debugFile('usleep:'.$waitTime);
                usleep( $waitTime );
            }
        }

        // Store random bit in file ... the microtime for example - might prove
        // useful some time.
        fwrite( $fp, microtime() );
        fclose( $fp );
        // Add first lock
        ++$this->lockLevel;
     }

    /**
     * Removes the lock.
     *
     * @return void
     */
    public function unlock() {
		debugFile('unlock:'.$this->lockLevel);
        if ( --$this->lockLevel === 0 )   {
            // Remove the lock file
            $lockFileName = $this->root . '/' . $this->lockFileName;
            unlink( $lockFileName );
			debugFile('unlockOK:'.$this->lockLevel);
        }
    }
    /**
     * Wait and get lock for complete directory tree.
     *
     * Acquire lock for the complete tree for read or write operations. This
     * does not implement any priorities for operations, or check if several
     * read operation may run in parallel. The plain locking should / could be
     * extended by something more sophisticated.
     *
     * If the tree already has been locked, the method waits until the lock can
     * be acquired.
     *
     * The optional second parameter $readOnly indicates wheather a read only
     * lock should be acquired. This may be used by extended implementations,
     * but it is not used in this implementation.
     *
     * @param bool $readOnly
     * @return void
     *
     * @todo The locking mechanism affects the ETag of the base collection. The
     *       ETag is different on each request, which might result in problems
     *       for clients that make extensive use of If-* headers. No client is
     *       known so far, if problems occur here we need to find a solution
     *       for this.
     */
    protected function acquireLock( $readOnly = false )
    {
        try
        {
            $this->lock(200000, 2000000 );
        }
        catch ( ezcWebdavLockTimeoutException $e )
        {
            return false;
        }
        return true;
    }

    /**
     * Free lock.
     *
     * Frees the lock after the operation has been finished.
     *
     * @return void
     */
    protected function freeLock()
    {
        $this->unlock();
    }


}
?>