<?php
/*
	+-----------------------------------------------------------------------------+
	| GROUP-E collaboration software                                              |
	+-----------------------------------------------------------------------------+
	| Copyright (c) 2002-2005 group-e,                                            |
	|         Endo7 GmbH/Srl                                                      |
	|	  via-Goethe-strasse 34a                                              |
	|	  39100 Bozen/Bolzano                                                 |
	|	  ITALIEN/ITALIA                                                      |
	|         contact@endo7.com                                                   |
	| This program is free software; you can redistribute it and/or               |
	| modify it under the terms of the GNU General Public License                 |
	| as published by the Free Software Foundation; either version 2              |
	| of the License, or (at your option) any later version.                      |
	|                                                                             |
	| This program is distributed in the hope that it will be useful,             |
	| but WITHOUT ANY WARRANTY; without even the implied warranty of              |
	| MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               |
	| GNU General Public License for more details.                                |
	|                                                                             |
	| You should have received a copy of the GNU General Public License           |
	| along with this program; if not, write to the Free Software                 |
	| Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. |
        | http://www.fsf.org/                                                         |
	+-----------------------------------------------------------------------------+
*/
//phpinfo();
require("cfg/global.inc.php");
require("lib/head_auth.php");
require("lib/parse_config.inc.php");


class geAutloader {
    /**
     * This variable stores all the elements from the autoload arrays. When a
     * new autoload file is loaded, their files are added to this array.
     *
     * @var array(string=>string)
     */
    protected static $autoloadArray = array();

    /**
     * The full path to the autoload directory.
     *
     * @var string
     */
    protected static $packages = array();

	public static function addPackage( $name, $path ) {
		self::$packages[$name]=$path;
	}
	/**
     * Tries to autoload the given className. If the className could be found
     * this method returns true, otherwise false.
     *
     * This class caches the requested class names (including the ones who
     * failed to load).
     *
     * @param string $className  The name of the class that should be loaded.
     *
     * @return bool
     */
    public static function autoload( $className ) {
        // Check whether the classname is already in the cached autoloadArray.
        if (array_key_exists( $className, self::$autoloadArray )) {
            // Is it registered as 'unloadable'?
            if ( self::$autoloadArray[$className] == false )  {
                return false;
            }
            self( self::$autoloadArray[$className] );
            return true;
        }

        // Not cached, so load the autoload from the package.
        // Matches the first and optionally the second 'word' from the classname.
        $fileNames = array();
		foreach (self::$packages as $pkName=>$pkDir) {
			if ( preg_match( "/^".($pkName)."([A-Z][a-z0-9]*)?([A-Z][a-z0-9]*)?([A-Z][A-Za-z0-9]*)?$/", $className, $matches ) ) {
			   switch (sizeof( $matches ))  {
					case 4:
						$autoloadFile = strtolower("{$matches[1]}/{$matches[2]}/").$className.'.php';
					break;
					case 3:
						$autoloadFile = strtolower("{$matches[1]}/").$className.'.php';
					break;
					case 2:
						$autoloadFile = $className.'.php';
					break;
				}
				if (self::loadFile( $pkDir,$autoloadFile)) {
					self::$autoloadArray[$className] = $autoloadFile;
					return true;
				}
				self::$autoloadArray[$className] = false;
			}
		}
        return false;
    }


    /**
     * Loads, require(), the given file name. If we are in development mode,
     * "/src/" is inserted into the path.
     *
     * @param string $file  The name of the file that should be loaded.
     */
    protected static function loadFile($pkDir, $file ) {
        if (file_exists( $pkDir . $file ))   {
           require( $pkDir . $file );
        } else  {
            // Can not be tested, because if this happens, then one of the
            // components has a broken autoload file.
            throw new Exception('FILE NOT FOUND:'. $pkDir.$file );
        }
	}
}

class Action {
	public $action;
	public $method;
	public $data;
	public $tid;
}

geAutloader::addPackage('ge','lib/');
spl_autoload_register(array( 'geAutloader', 'autoload' ));

//$T->loadModul('core');
$isForm = false;
$isUpload = false;
if (strpos($_SERVER["CONTENT_TYPE"],"application/json")!==false) {
	$postData=file_get_contents('php://input');
	$data = json_decode($postData);
	//debug($data);
	header('Content-Type: text/javascript');
} else if (isset($_REQUEST['extAction'])) { // form post
	$isForm = true;
	$isUpload = $_REQUEST['extUpload'] == 'true';
	$data = new Action();
	$data->extAction = $_REQUEST['extAction'];
	$data->extMethod = $_REQUEST['extMethod'];
    $data->extTID = isset($_REQUEST['extTID']) ? $_REQUEST['extTID'] : null; // not set for upload
	$data->data = array($_REQUEST, $_FILES);
} else {
	die('Invalid request.');
}
function doRpc($cdata){
	try {
		$action = $cdata->extAction;
		if(!class_exists($action)) {
			throw new Exception('Call to undefined class: ' .$action);
		}
		$o = new $action();

		$method = $cdata->extMethod;
		if(!method_exists($o,$method)){
			throw new Exception("Call to undefined method: $method on class $action");
		}

		$r = array(
			'type'=>'rpc',
			'tid'=>$cdata->extTID,
			'action'=>$action,
			'method'=>$method
		);

	    $params = isset($cdata->data) && is_array($cdata->data) ? $cdata->data : array();


		call_user_func_array(array($o, $method), $params);

		$r['result'] = $o->getResult();
	}
	catch(Exception $e){
		$r['type'] = 'exception';
		$r['message'] = $e->getMessage();
		$r['where'] = $e->getTraceAsString();
	}
	return $r;
}

$T->loadModul('core');
$response = null;
if (is_array($data)) {
	$response = array();
	foreach($data as $d){
		$response[] = doRpc($d);
	}
} else {
	$response = doRpc($data);
}
page_close();
if ($isForm && $isUpload) {
	echo '<html><body><textarea>';
	echo json_encode($response);
	echo '</textarea></body></html>';
} else {
	echo json_encode($response);
}

?>
